package connect4;

public class Board {

	/**
	 * Board - Constructor for connect4 playing field
	 * @return void
	 */
	public Board() {
		board = new int[42];
		for (int i = 0; i < 42;i++) {board[i] = 0;}
	}

	/**
	 * GET - Gets token, can return from anywhere on board;
	 * used by printCurrentBoard in Player(s) for debugging
	 * @param int location of token to get (0-41)
	 * @return void
	 */
	public int get(int x) {
		return board[x];
	}

	/**
	 * SET - Sets token on board
	 * @return void
	 */
	public void set(int tempPosition, int tempColor) {
		lastMove = tempPosition;
		drawCounter++;
		int temp = tempPosition +(7*5); // position at the bottom of a column
		//if(board[tempPosition] != 0) {return 0;}
		for (int j = 5;j >= 0; j--) {
			if(board[temp] == 0) {
				board[temp] = tempColor; // update the internal player board
				break;
			}
			temp -= 7; //move one up to check if ok to place token
		}
	}

	/**
	 * UNSET - Unsets last set token
	 * @return void
	 */
	public void unset() {
		drawCounter--;
		int temp = lastMove; // +(7*5); // position at the top of a column
		for (int j = 0;j <= 5; j++) {
			if(board[temp] != 0) {
				board[temp] = 0; // update the internal player board
				break;
			}
			temp += 7; //move one down to check if ok to place token
		}
	}

	/**
	 * GETTOKEN - Gets token from board
	 * @param column to get token from (0-6)
	 * @return token of either color red(1) or yellow(2)
	 */
	public int getToken(int iColumn) {
		int temp = iColumn; // position at the top of a column
		for (int j = 0;j <= 5; j++) {
			if(board[temp] != 0) {
				return board[temp];
			}
			temp += 7; //move one down to check if ok to get token
		}
		return 3; // not used
	}

	/**
	 * ISGAMEDRAWN - See if game is drawn
	 * @return true if 42 tokens is on board else false
	 */
	public Boolean isgameDrawn() { // returns 0 if game is drawn
		if (drawCounter >= 42) {return true;}
		return false;
	}

	/**
	 * WIN - See if 4 tokens connect
	 * @param color of player to check for - 1 for red, 2 for yellow
	 * @return 1 if 4 tokens connect, 0 if not
	 */
	public int win(int tempColor) { // returns 0 if game is drawn
		int playerColor = tempColor;
		int win=0;
		// CHECK ALL VERTICAL
		for (int i = 0; i <= 6; i++) {
			for (int j = i; j <= i+14; j+=7) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+7) == playerColor &&
					this.get(j+14) == playerColor &&
					this.get(j+21) == playerColor) {
					System.out.println("win VERT.");
					return win=1;
				}
			}
		}

		// CHECK ALL HORIZONTAL
		for (int i = 0; i <= 35; i+=7) {
			for (int j = i; j <= i+3; j++) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+1) == playerColor &&
					this.get(j+2) == playerColor &&
					this.get(j+3) == playerColor) {
					System.out.println("win HORI.");
					return win=1;
				}
			}
		}
		// CHECK ALL DIAGONALLY 1
		// FIRST FROM LOWER LEFT TO UPPER RIGHT
		if (this.get(14)==playerColor&&this.get(22)==playerColor &&
			this.get(30)==playerColor&&this.get(38)==playerColor) {System.out.println("win DIAG.1.1");return win=1;}
		if (this.get(7 )==playerColor&&this.get(15)==playerColor &&
			this.get(23)==playerColor&&this.get(31)==playerColor) {System.out.println("win DIAG.1.2");return win=1;}
		if (this.get(15)==playerColor&&this.get(23)==playerColor &&
			this.get(31)==playerColor&&this.get(39)==playerColor) {System.out.println("win DIAG.1.3");return win=1;}
		if (this.get(0 )==playerColor&&this.get( 8)==playerColor &&
			this.get(16)==playerColor&&this.get(24)==playerColor) {System.out.println("win DIAG.1.4");return win=1;}
		if (this.get( 8)==playerColor&&this.get(16)==playerColor &&
			this.get(24)==playerColor&&this.get(32)==playerColor) {System.out.println("win DIAG.1.5");return win=1;}
		if (this.get(16)==playerColor&&this.get(24)==playerColor &&
			this.get(32)==playerColor&&this.get(40)==playerColor) {System.out.println("win DIAG.1.6");return win=1;}
		if (this.get( 1)==playerColor&&this.get( 9)==playerColor &&
			this.get(17)==playerColor&&this.get(25)==playerColor) {System.out.println("win DIAG.1.7");return win=1;}
		if (this.get( 9)==playerColor&&this.get(17)==playerColor &&
			this.get(25)==playerColor&&this.get(33)==playerColor) {System.out.println("win DIAG.1.8");return win=1;}
		if (this.get(17)==playerColor&&this.get(25)==playerColor &&
			this.get(33)==playerColor&&this.get(41)==playerColor) {System.out.println("win DIAG.1.9");return win=1;}
		if (this.get( 2)==playerColor&&this.get(10)==playerColor &&
			this.get(18)==playerColor&&this.get(26)==playerColor) {System.out.println("win DIAG.1.10");return win=1;}
		if (this.get(10)==playerColor&&this.get(18)==playerColor &&
			this.get(26)==playerColor&&this.get(34)==playerColor) {System.out.println("win DIAG.1.11");return win=1;}
		if (this.get( 3)==playerColor&&this.get(11)==playerColor &&
			this.get(19)==playerColor&&this.get(27)==playerColor) {System.out.println("win DIAG.1.12");return win=1;}

		// THEN FROM UPPER LEFT TO LOWER RIGHT
		if (this.get( 3)==playerColor&&this.get( 9)==playerColor &&
			this.get(15)==playerColor&&this.get(21)==playerColor) {System.out.println("win DIAG.2.1");return win=1;}
		if (this.get( 4)==playerColor&&this.get(10)==playerColor &&
			this.get(16)==playerColor&&this.get(22)==playerColor) {System.out.println("win DIAG.2.2");return win=1;}
		if (this.get(10)==playerColor&&this.get(16)==playerColor &&
			this.get(22)==playerColor&&this.get(28)==playerColor) {System.out.println("win DIAG.2.3");return win=1;}
		if (this.get( 5)==playerColor&&this.get(11)==playerColor &&
			this.get(17)==playerColor&&this.get(23)==playerColor) {System.out.println("win DIAG.2.4");return win=1;}
		if (this.get(11)==playerColor&&this.get(17)==playerColor &&
			this.get(23)==playerColor&&this.get(29)==playerColor) {System.out.println("win DIAG.2.5");return win=1;}
		if (this.get(17)==playerColor&&this.get(23)==playerColor &&
			this.get(29)==playerColor&&this.get(35)==playerColor) {System.out.println("win DIAG.2.6");return win=1;}
		if (this.get( 6)==playerColor&&this.get(12)==playerColor &&
			this.get(18)==playerColor&&this.get(24)==playerColor) {System.out.println("win DIAG.2.7");return win=1;}
		if (this.get(12)==playerColor&&this.get(18)==playerColor &&
			this.get(24)==playerColor&&this.get(30)==playerColor) {System.out.println("win DIAG.2.8");return win=1;}
		if (this.get(18)==playerColor&&this.get(24)==playerColor &&
			this.get(30)==playerColor&&this.get(36)==playerColor) {System.out.println("win DIAG.2.9");return win=1;}
		if (this.get(13)==playerColor&&this.get(19)==playerColor &&
			this.get(25)==playerColor&&this.get(31)==playerColor) {System.out.println("win DIAG.2.10");return win=1;}
		if (this.get(19)==playerColor&&this.get(25)==playerColor &&
			this.get(31)==playerColor&&this.get(37)==playerColor) {System.out.println("win DIAG.2.11");return win=1;}
		if (this.get(20)==playerColor&&this.get(26)==playerColor &&
			this.get(32)==playerColor&&this.get(38)==playerColor) {System.out.println("win DIAG.2.12");return win=1;}
		return win;
	}

	/**
	 * WIN3 - See if 3 tokens connect
	 * @param color of player to check for - 1 for red, 2 for yellow
	 * @return 1 if 3 tokens connect, 0 if not
	 */
	public int win3(int tempColor) { // returns 0 if game is drawn
		int playerColor = tempColor;
		System.out.println("win3 playerColor: " +playerColor);
		int win=0;
		// CHECK ALL VERTICAL
		for (int i = 0; i <= 6; i++) {
			for (int j = i; j <= i+14; j+=7) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+7) == playerColor &&
					this.get(j+14) == playerColor) {
					win=1;
				}
			}
		}


		// CHECK ALL HORIZONTAL
		for (int i = 0; i <= 35; i+=7) {
			for (int j = i; j <= i+3; j++) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+1) == playerColor &&
					this.get(j+2) == playerColor) {
					win=1;
				}
			}
		}
		// CHECK ALL DIAGONALLY 1
		// FIRST FROM LOWER LEFT TO UPPER RIGHT
		if (this.get(14)==playerColor&&this.get(22)==playerColor &&
			this.get(30)==playerColor) {win=1;}
		if (this.get(7 )==playerColor&&this.get(15)==playerColor &&
			this.get(23)==playerColor) {win=1;}
		if (this.get(15)==playerColor&&this.get(23)==playerColor &&
			this.get(31)==playerColor) {win=1;}
		if (this.get(0 )==playerColor&&this.get( 8)==playerColor &&
			this.get(16)==playerColor) {win=1;}
		if (this.get( 8)==playerColor&&this.get(16)==playerColor &&
			this.get(24)==playerColor) {win=1;}
		if (this.get(16)==playerColor&&this.get(24)==playerColor &&
			this.get(32)==playerColor) {win=1;}
		if (this.get( 1)==playerColor&&this.get( 9)==playerColor &&
			this.get(17)==playerColor) {win=1;}
		if (this.get( 9)==playerColor&&this.get(17)==playerColor &&
			this.get(25)==playerColor) {win=1;}
		if (this.get(17)==playerColor&&this.get(25)==playerColor &&
			this.get(33)==playerColor) {win=1;}
		if (this.get( 2)==playerColor&&this.get(10)==playerColor &&
			this.get(18)==playerColor) {win=1;}
		if (this.get(10)==playerColor&&this.get(18)==playerColor &&
			this.get(26)==playerColor) {win=1;}
		if (this.get( 3)==playerColor&&this.get(11)==playerColor &&
			this.get(19)==playerColor) {win=1;}

		// THEN FROM UPPER LEFT TO LOWER RIGHT
		if (this.get( 3)==playerColor&&this.get( 9)==playerColor &&
			this.get(15)==playerColor) {win=1;}
		if (this.get( 4)==playerColor&&this.get(10)==playerColor &&
			this.get(16)==playerColor) {win=1;}
		if (this.get(10)==playerColor&&this.get(16)==playerColor &&
			this.get(22)==playerColor) {win=1;}
		if (this.get( 5)==playerColor&&this.get(11)==playerColor &&
			this.get(17)==playerColor) {win=1;}
		if (this.get(11)==playerColor&&this.get(17)==playerColor &&
			this.get(23)==playerColor) {win=1;}
		if (this.get(17)==playerColor&&this.get(23)==playerColor &&
			this.get(29)==playerColor) {win=1;}
		if (this.get( 6)==playerColor&&this.get(12)==playerColor &&
			this.get(18)==playerColor) {win=1;}
		if (this.get(12)==playerColor&&this.get(18)==playerColor &&
			this.get(24)==playerColor) {win=1;}
		if (this.get(18)==playerColor&&this.get(24)==playerColor &&
			this.get(30)==playerColor) {win=1;}
		if (this.get(13)==playerColor&&this.get(19)==playerColor &&
			this.get(25)==playerColor) {win=1;}
		if (this.get(19)==playerColor&&this.get(25)==playerColor &&
			this.get(31)==playerColor) {win=1;}
		if (this.get(20)==playerColor&&this.get(26)==playerColor &&
			this.get(32)==playerColor) {win=1;}
		return win;
	}

	/**
	 * GETLASTMOVE - Get last move made.
	 * @return last move
	 */
	public int getLastMove(){
		return lastMove;
	}

	// INSTANCE FIELD
	int[] board;
	int[] lastBoard;
	int drawCounter;
	int lastMove;
}
