package MagnusKoch;
import Player.Player;

import java.util.ArrayList;

public class MagnusKoch implements Player
{
	// Sets default name to Magnus
	private String name = "Magnus";
	
	// Color of the player. True if red.
	private boolean color;
	
	private SmartBoard myBoard;
	
	private final int POSSIBLE_MOVES = 6; // Including the move 0.
	
	/**
	 * Constructs a new MagnusKoch computer player
	 */
	public MagnusKoch()
	{
		myBoard = new SmartBoard();
	}
	
	/**
	 * Sets the color of the player. True if red.
	 * 
	 * @param color The players color
	 */
	public void init(Boolean color)
	{
		this.color = color;
	}
	
	/**
	 * Return the name of the player
	 * 
	 * @name Player name
	 */
	public String name()
	{
		return name;
	}
	
	/**
	 * Based on the moves previously made by the player, and the moves informed to the player the next move is calculated.
	 * @return move and int representing the column the move should be made to.
	 */
	public int move()
	{
		/**
		 * An inner class used to recurse through possible play situations.
		 *
		 */
		class BestMove
		{
			ArrayList<SmartBoard> boards = new ArrayList<SmartBoard>();
			SmartBoard board;
			int depth;
			boolean gameHasBeenWon;
			int bestMove;
			boolean redsTurn;
			
			BestMove(SmartBoard b, int depth, boolean redsTurn)
			{
				this.gameHasBeenWon = false;
				this.board = b;
				this.depth = depth;
				this.redsTurn = redsTurn;
				if(board.gameIsOver())
				{
					this.gameHasBeenWon = true;
				}
				else
				{
					for(int i = 0; i <= POSSIBLE_MOVES; i++)
						{
							if(board.spaceInColumn(i))
								boards.add(new SmartBoard(board.getBoardSituation(), i, redsTurn));
						}
				}
			}
			
			int max() 
			{	
				// If the game has been won, the recursion stops. Negative -100 is returned
				if(gameHasBeenWon) 						
					return -100; 
				// If the depth has been reached the board is evaluated. Evaluation value * -1 is returned.
				else if(this.depth == 0) 					 
				{
					if(board.lastTokenValue  == 2)
						return -25;
					else if(board.lastTokenValue == 3)
						return -50;
					else
						return 0;
				}
				else 									
				{
					this.bestMove = boards.get(0).getLastMove();
					int result = -101;
					// For each possible play on the current board an new BestMove object is made. Switching the turn
					// decrementing the depth, and switching from max to min.
					for(SmartBoard e: boards)
					{
						int minValue = new BestMove(e, depth -1, !redsTurn).min();
						if(minValue > result)
						{
							result = minValue;
							this.bestMove = e.getLastMove();
						}
						// Best possible result is 100. If this is reached the search stops.
						if(result >= 100) return result;
					}
					return result;
				}
			}
			
			int min()
			{
				if(gameHasBeenWon)
					return 100;
				else if(this.depth == 0)
				{
					if(board.lastTokenValue  == 2)
						return 25;
					else if(board.lastTokenValue == 3)
						return 50;
					else
						return 0;
				}
				else
				{
					this.bestMove = boards.get(0).getLastMove();
					int result = 101;
					for(SmartBoard e: boards)
					{
						int minValue = new BestMove(e, depth - 1, !redsTurn).max();
						if (minValue < result)
						{
							result = minValue;
							this.bestMove = e.getLastMove();
						}
						if(result <= -100) return result;
					}
					return result;
				}
			}
		}
		
		int depth = 7;
		BestMove gen = new BestMove(myBoard, depth, color);
		gen.max();
		int move = gen.bestMove;
		myBoard.inform(move);
		return move;	
	}
	
	/** informs the player about a new move
	 * 
	 * @i the column the move is made in
	 */
	public void inform(int i)
	{
		myBoard.informAndCheckIfWinner(i);
	}
	
}
