package connect4;

import javax.swing.JOptionPane;

public class PlayerHuman implements Player {

	/**
	 * PLAYERHUMAN - sets the name of a human player and inits
	 * a new board, could in time be used to implement an interface
	 * where the game had highscore..etc.
	 * @return void
	 */
	public PlayerHuman(String tempName) { // player is initialized with a name
		name = tempName;
		xBoard = new Board();
	}

	/**
	 * INFORM - is used to let the Connect4Viewer feed the data
	 * from a move back to the -opponent- of the player that made
	 * the current move
	 * @return void
	 */
	public void inform(int i) { // player is informed about the move of the opponent (by GUI or referee)
		if (playerColor == 1) {
			xBoard.set(i, 2);
		}
		else {
			xBoard.set(i, 1);
		}
	}

	/**
	 * SELFINFORM - is used to let the Connect4Viewer feed the data
	 * from a move back to the human players own board
	 * @return void
	 */
	public void selfInform(int i) { // player
		xBoard.set(i, playerColor);
	}

	/**
	 * INIT - initializes the player, true for red false for yellow,
	 * in this implementation this translates to 1 for red and 2
	 * for yellow
	 * @return void
	 */
	public void init(Boolean color) { // red is TRUE, yellow is FALSE
		if (color.equals(true)) {
			playerColor = 1;
		}
		else {
			playerColor = 2;
		}
	}

	/**
	 * MOVE - tells the player to move, not needed for human player
	 * that is perfectly able to move by simply looking at the screen
	 * @return 0 always
	 */
	public int move() { // player is asked to make a move (by GUI or referee)
		return 0;
	}

	/**
	 * PRINTCURRENTBOARD - prints current board to the console for debugging
	 * @return void
	 */
	public void printCurrentBoard(){
		System.out.println("playercolor_" + playerColor + "has so board yo:");
		System.out.println("|   "+ xBoard.get(0)+"|   "+xBoard.get(1)+"   "+xBoard.get(2)+"|   "+xBoard.get(4)+"|   "+xBoard.get(4)+"|   "+xBoard.get(5)+"|   "+xBoard.get(6)+"|");
		System.out.println("|   "+ xBoard.get(7)+"|   "+xBoard.get(8)+"   "+xBoard.get(9)+"|   "+xBoard.get(10)+"|   "+xBoard.get(11)+"|   "+xBoard.get(12)+"|   "+xBoard.get(13)+"|");
		System.out.println("|   "+ xBoard.get(14)+"|   "+xBoard.get(15)+"   "+xBoard.get(16)+"|   "+xBoard.get(17)+"|   "+xBoard.get(18)+"|   "+xBoard.get(19)+"|   "+xBoard.get(20)+"|");
		System.out.println("|   "+ xBoard.get(21)+"|   "+xBoard.get(22)+"   "+xBoard.get(23)+"|   "+xBoard.get(24)+"|   "+xBoard.get(25)+"|   "+xBoard.get(26)+"|   "+xBoard.get(27)+"|");
		System.out.println("|   "+ xBoard.get(28)+"|   "+xBoard.get(29)+"   "+xBoard.get(30)+"|   "+xBoard.get(31)+"|   "+xBoard.get(32)+"|   "+xBoard.get(33)+"|   "+xBoard.get(34)+"|");
		System.out.println("|   "+ xBoard.get(35)+"|   "+xBoard.get(36)+"   "+xBoard.get(37)+"|   "+xBoard.get(38)+"|   "+xBoard.get(39)+"|   "+xBoard.get(40)+"|   "+xBoard.get(41)+"|");
	}

	/**
	 * ISDRAWNCHECK - checks if there is a win on the board associated
	 * to the player, relies on "isgameDrawn" function in board class
	 * @return void
	 */
	public void isDrawnCheck(){
		if (xBoard.isgameDrawn()) {
			JOptionPane.showMessageDialog(null, "Game is over, no winner.",
			"Game drawn", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * WIN - checks if there is a win on the board associated
	 * to the player, relies on "win" function in board class
	 * @return 1 if win, 0 if not
	 */
	public int win() { // player testing whether its a win
		int win=0;
		win = xBoard.win(playerColor);
		return win;
	}

	/**
	 * RESET - makes a new board for the player
	 * @return void
	 */
	public void reset() { // return the name of the player
		xBoard = new Board();
	}

	/**
	 * NAME -  returns the name of the player
	 * @return name, string
	 */
	public String name() { //
		return name;
	}
	// INSTANCE FIELD
	String name;
	Board xBoard;
	int playerColor;
	int win = 0;
}
