/* Connect 4 */
/* Carsten Schuermann */

package CarstenSchuermann;

import java.util.ArrayList;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import Player.Player;
import java.util.Random;

class IllegalMove extends Exception {
    int i; 
    IllegalMove (int i) {
	this.i = i;
    }
}

class Board {
    Boolean[][] board;
 
    Board () {
	init ();
    }

    void init () {
	board = new Boolean[7][6];
    }

    Board copy () { 
	Board newboard = new Board();
	for (int i=0; i<7; i++) {
	    for (int j=0; j<6; j++) {
		newboard.board[i][j]= board[i][j];
	    }
	}
	return newboard;
    }
	

    Boolean isValid (int i) {
	if (0 <= i && i < 7) {return true;}
	else {return false;}
    }

    Boolean isFull (int i) {
	if (board[i][5] == null) {return false;} 
	else {return true;}
    }

    void drop (int i, int j, Boolean color) {
	if (board[i][j] == null) {
	    if (j == 0) { board[i][j] = color;}
	    else { this.drop (i, j-1, color);}
	}
	else {board[i][j+1] = color;}      /* will be ok by invariant */
    }

    void insert (int i, Boolean color) throws IllegalMove {
	if (this.isFull(i)) {throw (new IllegalMove (i));} 
	else {
	    if (this.isValid(i)) {this.drop (i, 5, color);}
	    else {throw (new IllegalMove (i));};	    
	}
    }

    Boolean slash(Boolean color, int i, int j) {
	if (i>3 || j>2) {return false;}
	else {
	    Boolean b = true;
	    for (int k = 0; k < 4; k++)
		{b = b && (board[i+k][j+k]) == color; }
	    return b;
	}
    }
	
    Boolean minus(Boolean color, int i, int j) {
	if (i>3) {return false;}
	else {
	    Boolean b = true;
	    for (int k = 0; k < 4; k++)
		{b = b && (board[i+k][j]) == color; }
	    return b;
	}
    }
	
    Boolean backslash(Boolean color, int i, int j) { 
	if (i>3 || j<3) {return false;}
	else {
	    Boolean b = true;
	    for (int k = 0; k < 4; k++)
		{b = b && (board[i+k][j-k]) == color; }
	    return b;
	}
    }

    Boolean pipe (Boolean color, int i, int j) {
	if (j<3) {return false;}
	else {
	    Boolean b = true;
	    for (int k = 0; k < 4; k++)
		{b = b && (board[i][j-k]) == color; }
	    return b;
	}
    }

    Boolean win (Boolean color) {
	Boolean b = false;
	for (int i = 0 ; i<7 ; i++) {
	    for (int j = 0; j< 6; j++) {
		b = b || slash (color, i, j) 
		    || minus (color, i, j) 
		    || backslash (color, i, j)
		    || pipe (color, i, j);
	    }
	}
	return (b);
    }

    Boolean tie () {
	Boolean b = true;
	for (int i = 0; i<7; i++) {
	    b = b && isFull(i);
	}
	return b;
    }
}

class Stage {
    
    Board[] boards = new Board[7];
    Boolean color;
    int best;
    Boolean winning;                          /* is never null */
    Boolean loosing;                          /* is never null */
    int depth;

    Random r = new Random ();

    Stage (Board b, Boolean color, int depth) {
	this.depth = depth;
	if (b.win (! color)) {loosing = true; winning = false;}
	else { 
	    if (b.win (color)) {winning = true; loosing = false;}
	    else {
		loosing = false;
		winning = false;
		this.color = color;
		for (int i=0; i<7; i++) {		   
		    if (b.isFull (i)) {boards[i] = null;}
		    else{
			boards[i] = b.copy ();
 			try {boards[i].insert(i, color);} catch (IllegalMove exn) {};
 		    }
 		}
	    }
	}	
    }

    double minimax (double a, double b) {
	if (this.depth < 0) {return 0.0;} 
	if (this.winning) {return +1.0;}
	if (this.loosing) {return -1.0;}
	else {
	    double newa = a;
	    best = 0;
	    for (int i = 0; i<7; i++) {
		if (boards[i] != null) { 
		    double n = new Stage (boards[i], ! color, this.depth-1).maximin (a, b);
		    if (newa < n) {newa = n; best = i;}
		    if (b <= newa) {return newa;}
		}
	    }
	    return newa;
	}
    }

    double maximin (double a, double b) {
	if (this.depth < 0) {return 0.0;} 
	if (winning) {return -1.0;}
	if (loosing) {return 1.0;}
	else {
	    double newb = b;
	    best = 0;
	    for (int i=0; i<7; i++) {
		if (boards[i] != null) { 
		    double n = new Stage (boards[i], ! color, this.depth-1).minimax (a, b);
		    if (n < newb) {newb = n; best = i;}
		    if (newb <= a) {return newb;}
		}
	    }
	    return newb;
	}
    }
}




public class CarstenSchuermann implements Player {
    Board board = new Board ();
    Boolean color;
    
    public String name () { return "Carsten";}

    public void init (Boolean color) {
	this.color = color;
    }

    public int move () { 
	Stage start= new Stage (board, color, 5);
	start.minimax (-100.0, 100.0);
	try {board.insert (start.best, color);} catch (IllegalMove exn) {};
	return (start.best);
    }

    public void inform (int i) {
	try {board.insert (i, ! color);} catch (IllegalMove exn) {};
    }
   
}
