package ChristianFoli;
import java.util.ArrayList;
import Player.Player;

public class ChristianFoli implements Player{

	public ChristianFoli() {
		name = "Mechanical Christian";
		board = new Board();
	}

	public ChristianFoli(String aName) {
		name = aName;
		board = new Board();
	}

	public void init(Boolean color) {
		//1 = RED, 2 = YELLOW
		if(color == false) {
			myColorInt = 2;
			otherColorInt = 1;
		}
		else {
			myColorInt = 1;
			otherColorInt = 2;
		}
		board.printBoard();
	}

	public String name() {
		return name;
	}

	public int move() {
		//if first move, play in the middle
		if(board.totalMoves < 2) {
			board.setField(3, myColorInt);
			return 3;
		}
		else {
			return max();
		}
	}

	/*
	 * Intended to decide the best move based on a set of rules
	 * assigning points to the individual columns. AI (hopefully)
	 * won't play a field that allows the opponent to win in next
	 * go, but other than that it can't look ahead.
	 * But it still plays ok...
	 */
	private int max() {
		int[] moves = new int[7];
		for(int i = 0; i < 7; i++)  {
			int field = board.getNextFree(i);

			int columnValue = 0;
			if(field > 7) {
			columnValue = columnValue + board.canWin(field, myColorInt);
			columnValue = columnValue + board.canBlockWin(field, otherColorInt);
			columnValue = columnValue + board.canWinInNext(field, myColorInt);
			columnValue = columnValue + board.canBlockWinInNext(field, otherColorInt);
			columnValue = columnValue + board.canGetObscureThreat(field, myColorInt);
			columnValue = columnValue + board.canBlockObscureThreat(field, otherColorInt);
			columnValue = columnValue + board.canGetThreat(field, myColorInt);
			columnValue = columnValue + board.canBlockThreat(field, otherColorInt);
			columnValue = columnValue + board.canGetImminentThreat(field, myColorInt);
			columnValue = columnValue + board.canBlockImminentThree(field, otherColorInt);
			columnValue = columnValue + board.canGetTwo(field, myColorInt);
			columnValue = columnValue + board.canBlockTwo(field, otherColorInt);

			board.setField(i, myColorInt);
			int nextField = board.getNextFree(i);

			if(board.canWin(nextField, otherColorInt) > 0){
				columnValue = -1;
				System.out.println("Column to -1");}
			if(board.canWin(nextField, otherColorInt) > 0){
				columnValue = 0;
				System.out.println("Column to 0");}

			board.undoMoves(1);

			moves[i] = columnValue;
			}

			else {
				moves[i] = -1;
			}
		}

		int bestColumn = 0;
		int highestValue = 0;
		for(int i = 0; i < 7; i++) {
			System.out.println("Column[" + i + "]: " + moves[i]);
			System.out.println("Field: " + board.getNextFree(i));
			if(moves[i] >= highestValue) {
				highestValue = moves[i];
				bestColumn = i;
			}
		}
		board.setField(bestColumn, myColorInt);
		board.printBoard();
		return bestColumn;

	}

	public void inform(int move){
		board.setField(move, otherColorInt);
		}


	public void reset() {
		board = new Board();
	}

	private String name;
	public int myColorInt;
	private int otherColorInt;
	private Board board;
}



/*
The array represeneting the board:

	! 0  1  2  3  4  5  6! 7
	+--------------------+---
	! 8  9 10 11 12 13 14!15
	!16 17 18 29 20 21 22!23
	!24 25 26 27 28 29 30!31
	!32 33 34 35 36 37 38!39
	!40 41 42 43 44 45 46!47
	!48 49 50 51 52 53 54!55
	+--------------------+---
	!56 57 58 59 60 61 62!63
	!64 65 66 67 68 69 70!71 <- holds value for next free

	The initialised gameboard
	-1 ensures no false positives are found
	when checking for 4 connected

   |-1 |-1 |-1 |-1 |-1 |-1 |-1 |-1
   +---------------------------+--
   |0  |0  |0  |0  |0  |0  |0  |-1
   |0  |0  |0  |0  |0  |0  |0  |-1
   |0  |0  |0  |0  |0  |0  |0  |-1
   |0  |0  |0  |0  |0  |0  |0  |-1
   |0  |0  |0  |0  |0  |0  |0  |-1
   |0  |0  |0  |0  |0  |0  |0  |-1
   ----+---------------------------+--
   |-1 |-1 |-1 |-1 |-1 |-1 |-1 |-1
   |6  |6  |6  |6  |6  |6  |6  |-1  <- holds value for next free

*/
class Board implements Cloneable {

	public Board() {

		boardFields = new int [72];
		for(int i = 0; i < 72; i++)
			boardFields[i] = -1; //INITIALISE ALL FIELDS TO -1
		for(int j = 0; j < 7; j++) {
			for(int k = 0; k < 6; k++)
				boardFields[(8+j)+(k*8)] = 0; //INITIALISE ALL PLAYABLE FIELDS TO 0
		}
		for(int l = 64; l < 71; l++)
			boardFields[l] = 6; //THE NEXT FREE SQUARE IN THE COLUMN

		directionsArray[0] = WEST;
		directionsArray[1] = EAST;
		directionsArray[2] = SOUTH;
		directionsArray[3] = SOUTHEAST;
		directionsArray[4] = SOUTHWEST;
		directionsArray[5] = NORTHEAST;
		directionsArray[6] = NORTHWEST;

		ArrayList<Integer> player1ThreatList = new ArrayList<Integer>();
		ArrayList<Integer> player2ThreatList = new ArrayList<Integer>();
		threatLists = new ArrayList[3];
		threatLists[1] = player1ThreatList;
		threatLists[2] = player2ThreatList;

		undoList = new int[42];
		for(int i = 0; i < 42; i++)
			undoList[i] = 0; //fill the undolist w. 0's

	}

	public void setBoard(int[] simBoard) {
		boardFields = simBoard;
	}

	/*
	 * Sets the played field
	 */

	public void setField(int move, int player) {

		//THE NEXT EMPTY SQUARE IS column + (nextfree *8)
		//ex. 0 + (6*8) = 48, the first empty square in column 0
		//nextfree is contained in boardFields[column + 64]
		int field = move + (boardFields[move + 64] * 8);
		int emptyCounter = move + 64; //decrement the amount of empty squares in the column
		boardFields[emptyCounter] = boardFields[emptyCounter]-1; //decrement the amount of free squares
		boardFields[field] = player;
		totalMoves++;
		undoList[totalMoves-1] = field;
	}


	/*
	 * CHECKS IF THE GAME IS WON
	 * Takes its starting point in the last move.
	 * Throws ArrayIndexOutOfBoundsException only when
	 * checking NORTHWEST (-9) in top field of column 0
	 * (8-9 = -1). As NORTHWEST is last direction which is
	 * checked, and it is not possible to connect4 in northern
	 * direction from the top row, it is safe to return false
	 * if no win involving field 8 has been found yet.
	 *
	 */
	public boolean checkWin(int player, int lastmove) throws ArrayIndexOutOfBoundsException{

		int lastfield = lastmove + (boardFields[lastmove + 64] + 1) *8 ;
			for(int direction : directionsArray) {
				try {
					if(boardFields[lastfield] == player && boardFields[lastfield] == boardFields[lastfield + direction]) { //FOUND TWO IN A ROW

						if(checkWinHelper(lastfield + direction, direction)) //FOUND THREE IN A ROW

							//CHECKING FOR NEIGHBOUR'S NEIGHBOUR'S NEIGHBOUR OR NEIGHBOUR IN OPPOSITE DIRECTION
							if(checkWinHelper(lastfield + (2*direction), direction) ||
									checkWinHelper(lastfield - direction, direction)) {
								return true;}
						}
					}
				catch(ArrayIndexOutOfBoundsException e) {
					return false;
				}
			}

		return false;
		}

	/*
	 * HELPER METHOD FOR CHECKWIN
	 */
	public boolean checkWinHelper(int field, int direction) {


			if(boardFields[field] == boardFields[field + direction]) {
				return true;
			}
		return false;
		}

	/*
	 * Print boards for debugging
	 */

	public void printBoard() {
//		System.out.println(player + "'s current board:");
		System.out.println(this.toString());
		for(int i = 0; i < 72; i+= 8) {
			for(int j = 0; j < 8; j++)
				System.out.print("    |" + boardFields[i+j]);
			System.out.println();
		}
		System.out.println();
		System.out.println();
	}

	public int[] getBoard() {
		return boardFields;
	}

	/*
	 * Undo an amount of moves
	 * (hopefully) used for simulation of moves for AI
	 */
	public void undoMoves(int amountOfMoves) {
		for(int i = amountOfMoves; i > 0; i--) {
			System.out.println("TotalMoves: " + totalMoves);
			System.out.println("Undo List: " + undoList[totalMoves-1]);
			boardFields[undoList[totalMoves-1]] = 0;
			boardFields[undoList[totalMoves-1]%8 + 64]++;
			totalMoves--;
		}

	}

	/* ########################################################
	 * ##													 ##
	 * ##	BELOW ARE EVALUATION METHODS USED TO ASSIGN		 ##
	 * ##	A VALUE TO ALL PLAYABLE FIELDS. USED BY AI.		 ##
	 * ##													 ##
	 * ########################################################
	 *
	 * They all throw ArrayIndexOutOfBoundsException. However,
	 * this can only occur when checking NORTHWEST (-9) on the
	 * top left field of board (8 - 9 = -1). Therefore, it's
	 * safe to just return the value of the field when the
	 * exception is raised. It will not devalue the field.
	 */

	/*
	 * Returns the next empty square in a column
	 */
	public int getNextFree(int column) {
		return boardFields[column + 64] * 8 + column;
	}


	/*
	 * Can AI player win in next move
	 */

	public int canWin(int field, int player) throws ArrayIndexOutOfBoundsException {
		try {
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player) {
					if(boardFields[field + direction] == boardFields[field + 2*direction] &&
							boardFields[field + 2*direction] == boardFields[field + 3*direction]) {
						return 10000;
						}
					if(boardFields[field + direction] == boardFields[field + 2*direction] &&
							boardFields[field + direction] == boardFields[field - direction]) {;
						return 10000;
					}
				}
			}
		}
		catch(ArrayIndexOutOfBoundsException e) {
			return 0;
		}
		return 0;
	}

	/*
	 * Can AI block a win in next move
	 */

	public int canBlockWin(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try {
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player) {
					if(boardFields[field + direction] == boardFields[field + 2*direction] &&
							boardFields[field + direction] == boardFields[field + 3*direction]) {
							value = value + 1000;
							}
					if(boardFields[field + direction] == boardFields[field + 2*direction] &&
							boardFields[field + direction] == boardFields[field - direction] &&
							boardFields[field + direction] == player) {
							value = value + 1000;
						}
				}
			}
		}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI connect three with two imminent threats?
	 */
	public int canWinInNext(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player) {
					if(boardFields[field + direction] == boardFields[field + 2*direction]) {
						if(boardFields[field + 3*direction] == 0 && boardFields[field - direction] == 0) {
							value = value + 600;
							}
					}
				}
			}
		}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI block a connect three with two imminent threats?
	 */
	public int canBlockWinInNext(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player) {
					if(boardFields[field + direction] == player && boardFields[field + 2*direction] == player) {
						if(boardFields[field + 3*direction] == 0) {
							value = value + 700;
							}
					}
				}
			}
		}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI get an obscure semi-threat.
	 * place two piece on line with an empty space between and room for connect 4
	 */
	public int canGetObscureThreat(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
					if(boardFields[field + direction] == player && boardFields[field + 2*direction] == 0) {
						if(boardFields[field + 3*direction] == player && boardFields[field + 4*direction] == 0) {
							if(isPlayable(field) || isPlayable(field + 2*direction) || isPlayable(field + 4*direction))
							value = value + 400;
							}
					}
				}
			}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI block an obscure semi-threat.
	 */

	public int canBlockObscureThreat(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
					if(boardFields[field + direction] == player && boardFields[field + 2*direction] == 0) {
						if(boardFields[field + 3*direction] == player && boardFields[field + 4*direction] == 0) {
							if(isPlayable(field) || isPlayable(field + 2*direction) || isPlayable(field + 4*direction))
							value = value + 300;
							}
					}
				}
			}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI connect three
	 */

	public int canGetThreat(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player && boardFields[field + 2*direction] == player) {
					if(boardFields[field + 3*direction] == 0 || boardFields[field - direction] == 0) {
							if(value > 199) { //if more than one threat by playing a field, award extra points!
								value = value + 100;}
							System.out.println("canGetThreat in column " + field%8);
							value = value + 200;
							}
					}
				}
			}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI block a connect three
	 */

	public int canBlockThreat(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
					if(boardFields[field + direction] == player && boardFields[field + 2*direction] == player) {
						if(boardFields[field + 3*direction] == 0) {
							if(value > 149) { //if more than one threat can be avoided, award extra points
								value = value + 100;}
							value = value + 150;
							}
					}
				}
			}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI get a connect three that is immediately playable
	 * (obtaining two of these by playing one field is equal to
	 * winning the game in next go)
	 */

	public int canGetImminentThreat(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player && boardFields[field + 2*direction] == player) {
					if(boardFields[field + 3*direction] == 0 && isPlayable(field + 3*direction)) {
							if(value > 99) { //if more than one imminent threat by playing a field, game practically won!
								value = value + 700;}
							value = value + 100;
							}
					}
				}
			}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI block a connect three that is immediately playable
	 */
	public int canBlockImminentThree(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player && boardFields[field + 2*direction] == player) {
					if(boardFields[field + 3*direction] == 0 && isPlayable(field + 3*direction)) {
							if(value > 99) { //if more than one imminent threat can be avoided, award extra points
								value = value + 100;}
							value = value + 100;
							}
					}
				}
			}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI connect two
	 */
	public int canGetTwo(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player && boardFields[field + 2*direction] == 0
						&& boardFields[field + 3*direction] == 0) {
					if(value > 49){ //extra points for connecting in several directions
						value = value + 50;}
					value = value + 50;
						}
					}
				}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}

	/*
	 * Can AI block a connect two
	 * (virtually worthless move, but better than playing
	 * a column worth zero)
	 */

	public int canBlockTwo(int field, int player) throws ArrayIndexOutOfBoundsException {
		int value = 0;
		try{
			for(int direction : directionsArray) {
				if(boardFields[field + direction] == player && boardFields[field + 2*direction] == 0
						&& boardFields[field + 3*direction] == 0) {
					if(value > 9){ //extra points for block in several directions
						value = value + 10;}
					value = value + 10;
						}
					}
				}
		catch(ArrayIndexOutOfBoundsException e) {
			return value;
		}
		return value;
	}


	/*
	 * Determines if an empty field is playable in next go
	 */
	public boolean isPlayable(int field) {
		int column = field%8;
		if(column + (boardFields[column + 64] * 8) == field) {
			System.out.println(field + " is immediately playable");
			return true;
		}
		return false;
	}


	private int boardFields[];
	private int[] directionsArray  = new int[7];
	private ArrayList[] threatLists;
	private int[] undoList;

	public int totalMoves;

//	DIRECTIONS
	int EAST = 1;
	int WEST = -1;
	int SOUTH = 8;
	int SOUTHWEST = 7;
	int SOUTHEAST = 9;
	int NORTHWEST = -9;
	int NORTHEAST = -7;
}
