package Dajana;

public class Board
{
    private static final int ROW_SIZE = 6;
    private static final int COL_SIZE = 7;
    
    private Boolean[][] board;
    private Boolean player;
    private int column, row;

    public Board()
    {
        board = new Boolean[ROW_SIZE][COL_SIZE];        
    }
    
    public Board(Board b)
    {
        board = new Boolean[ROW_SIZE][COL_SIZE];
        Boolean[][] brd = b.getBoardArray();
        for(int i = 0; i < ROW_SIZE; i++)
        {
            for (int j = 0; j < COL_SIZE; j++)
            {
                board[i][j] = brd[i][j];
            }
        }
    }
    
    public Boolean[][] getBoardArray()
    {
        return board;
    }
    
    public void insert(int column, Boolean player) 
    {
        this.column = column;
        this.player = player;
        
        //System.out.println("insert called");
        
        if (isValid(this.column))
        {
            for (int i = ROW_SIZE - 1; i >= 0; i--)
            {
                if (board[i][this.column] == null)
                {
                    board[i][this.column] = this.player;
                    row = i;
                    //System.out.println("inserted");
                    break;
                }
            }
        }
        else
        {
            System.out.println("illegal move");
        }
    }
    
    public boolean isValid(int move)
    {
        if (board[0][move] == null)            
            return true;
        else return false;
    }
    
    public boolean tie()
    {
        for (int i = 0; i < ROW_SIZE; i++)
        {
            for (int j = 0; j < COL_SIZE; j++)
            {
                if (board[i][j] == null)
                    return false;
            }
        }
        return true;
    }
    
    public boolean win()
    {
        int count;
        count = 1;
        //vertical check
        for (int i = row + 1; i <= ROW_SIZE - 1; i++)
        {
            if (board[i][this.column] == this.player)
            {
                count += 1;
                //System.out.println("COUNT: " + count);
            }
            else 
                break;
        }
        
        //horizontal check
        if (count != 4)
        {
            count = 1;
            for (int i = this.column - 1; i >= 0; i--)
            {
                if (board[row][i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
             }
             
             for (int i = this.column + 1; i < COL_SIZE; i++)
             {
                if (board[row][i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
             }
         }
         
        //diagonal check from left to right
         if (count != 4)
        {
            count = 1;
            for (int i = 1; (0 <= row - i) && (0 <= this.column - i); i++)
            {
                if (board[row - i][this.column - i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
                
             for (int i = 1; (ROW_SIZE > row + i) && (COL_SIZE > this.column + i); i++)
            {
                if (board[row + i][this.column + i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
         }
         
         //diagonal check from right to left
         if (count != 4)
        {
            count = 1;
            for (int i = 1; (ROW_SIZE > row + i) && (0 <= this.column - i); i++)
            {
                if (board[row + i][this.column - i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
                
             for (int i = 1; (0 <= row - i) && (COL_SIZE > this.column + i); i++)
            {
                if (board[row - i][this.column + i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
         }
        
        //System.out.println("Count: " + count);
        
        if (count == 4)
        {
            //System.out.println("You've won");
            return true;
        }
        else return false;
    }
    
    public int getRowSize()
    {
        return ROW_SIZE;
    }
    
    public int getColumnSize()
    {
        return COL_SIZE;
    }
    
    public Boolean getToken(int row, int column)
    {
        int x = row;
        int y = column;
        Boolean token = board[x][y];
        return token;
    }
    
    public int getMove()
    {
        return this.column;
    }
    
    public String toString()
    {
        String str = "";
        for (int i = 0; i < ROW_SIZE; i++)
        {
            for (int j = 0; j < COL_SIZE; j++)
            {
                if (board[i][j] == null)
                    str += "-";
                else if (board[i][j] == true)
                    str += "1";
                else if (board[i][j] == false)
                    str += "2";
            }
            str += "\n";
        }
        return str;
    }
    
    private class IllegalMove extends Exception 
    {
        private int column; 
        public IllegalMove (int column) 
        {
            this.column = column;
        }
    }
}
