/* Connect 4 */

package Dajana;

import Player.Player;
import java.util.Iterator;
import java.util.ArrayList;


public class Dajana implements Player
{
    private static final int ROW_SIZE = 6;
    private static final int COL_SIZE = 7;

    private String name;
    private int move, column, row, checkMove;
    private Boolean color, player;
    private Board AIboard;       

    public Dajana()
    {
        name = "Dajana";
        AIboard = new Board();
    }
    
    public void init (Boolean color)
    {
        this.color = color;
    }
    
    public String name()
    {        
        return name;
    }
    
    public int move()
    {
            int depth = 10;
            //System.out.print("AI board: \n" + AIboard);
            MinMax node = new MinMax(AIboard, depth, this.color);
            node.max(-100, 100);
            int move = node.getBest();
            AIboard.insert(move, this.color);
        return move;
    }
    
    public void inform(int i)
    {
        AIboard.insert(i, !this.color);
    }
    
    public void setMove(int column)
    {
    }           
} 


class Board
{
    private static final int ROW_SIZE = 6;
    private static final int COL_SIZE = 7;
    
    private Boolean[][] board;
    private Boolean player;
    private int column, row;

    public Board()
    {
        board = new Boolean[ROW_SIZE][COL_SIZE];        
    }
    
    public Board(Board b)
    {
        board = new Boolean[ROW_SIZE][COL_SIZE];
        Boolean[][] brd = b.getBoardArray();
        for(int i = 0; i < ROW_SIZE; i++)
        {
            for (int j = 0; j < COL_SIZE; j++)
            {
                board[i][j] = brd[i][j];
            }
        }
    }
    
    public Boolean[][] getBoardArray()
    {
        return board;
    }
    
    public void insert(int column, Boolean player) 
    {
        this.column = column;
        this.player = player;
        
        //System.out.println("insert called");
        
        if (isValid(this.column))
        {
            for (int i = ROW_SIZE - 1; i >= 0; i--)
            {
                if (board[i][this.column] == null)
                {
                    board[i][this.column] = this.player;
                    row = i;
                    //System.out.println("inserted");
                    break;
                }
            }
        }
        else
        {
            System.out.println("illegal move");
        }
    }
    
    public boolean isValid(int move)
    {
        if (board[0][move] == null)            
            return true;
        else return false;
    }
    
    public boolean tie()
    {
        for (int i = 0; i < ROW_SIZE; i++)
        {
            for (int j = 0; j < COL_SIZE; j++)
            {
                if (board[i][j] == null)
                    return false;
            }
        }
        return true;
    }
    
    public boolean win()
    {
        int count;
        count = 1;
        //vertical check
        for (int i = row + 1; i <= ROW_SIZE - 1; i++)
        {
            if (board[i][this.column] == this.player)
            {
                count += 1;
                //System.out.println("COUNT: " + count);
            }
            else 
                break;
        }
        
        //horizontal check
        if (count != 4)
        {
            count = 1;
            for (int i = this.column - 1; i >= 0; i--)
            {
                if (board[row][i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
             }
             
             for (int i = this.column + 1; i < COL_SIZE; i++)
             {
                if (board[row][i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
             }
         }
         
        //diagonal check from left to right
         if (count != 4)
        {
            count = 1;
            for (int i = 1; (0 <= row - i) && (0 <= this.column - i); i++)
            {
                if (board[row - i][this.column - i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
                
             for (int i = 1; (ROW_SIZE > row + i) && (COL_SIZE > this.column + i); i++)
            {
                if (board[row + i][this.column + i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
         }
         
         //diagonal check from right to left
         if (count != 4)
        {
            count = 1;
            for (int i = 1; (ROW_SIZE > row + i) && (0 <= this.column - i); i++)
            {
                if (board[row + i][this.column - i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
                
             for (int i = 1; (0 <= row - i) && (COL_SIZE > this.column + i); i++)
            {
                if (board[row - i][this.column + i] == this.player)
                {
                    count += 1;
                    //System.out.println("COUNT: " + count);
                }
                else 
                    break;
                }
         }
        
        //System.out.println("Count: " + count);
        
        if (count == 4)
        {
            //System.out.println("You've won");
            return true;
        }
        else return false;
    }
    
    public int getRowSize()
    {
        return ROW_SIZE;
    }
    
    public int getColumnSize()
    {
        return COL_SIZE;
    }
    
    public Boolean getToken(int row, int column)
    {
        int x = row;
        int y = column;
        Boolean token = board[x][y];
        return token;
    }
    
    public int getMove()
    {
        return this.column;
    }
    
    public String toString()
    {
        String str = "";
        for (int i = 0; i < ROW_SIZE; i++)
        {
            for (int j = 0; j < COL_SIZE; j++)
            {
                if (board[i][j] == null)
                    str += "-";
                else if (board[i][j] == true)
                    str += "1";
                else if (board[i][j] == false)
                    str += "2";
            }
            str += "\n";
        }
        return str;
    }
    
    private class IllegalMove extends Exception 
    {
        private int column; 
        public IllegalMove (int column) 
        {
            this.column = column;
        }
    }
}
//*****************************************************************************************//  

/**
 * MinMax represents a minmax ai algorithm, which is implemented as a game tree. 
 * The constructor creates a node that represents the board position. When the max method is called on that node, 
 * it looks at all its children which are legal moves from that position and calls the min method on the the children of the children.
 * Each position returns a max value which is favorable for the player and in this case it is 1.
 * The min value that is returned it is favorable for the opposite player and in this case it is - 1.
 * Finally 0 is the value that is returned if the move is not from a favorable value for any of the players.
 */
class MinMax
{
    private Board brd, newBoard;
    private int depth, best;
    private Boolean player;
    private ArrayList<Board> boards;
    
    /**
     * Constructs nodes for each ligal move made on the current board.
     * @param board is the current Board that the players play on.
     * @player is a the color of the player making the move.
     */
    public MinMax(Board board, int depth, Boolean player)
    {
        brd = board;
        this.depth = depth;
        this.player = player;
        boards = new ArrayList<Board>();
        //System.out.println("Constractor Depth: " + this.depth);
        if (brd.win()) 
        {
            //System.out.println("==================Game Over");
        }
        else 
        {
            for (int n = 0; n < brd.getColumnSize(); n++) 
            {
                //System.out.print("Old board: \n" + brd);
                newBoard = new Board(brd);
                //System.out.print("New board: \n" + newBoard);
                if(newBoard.isValid(n))
                {
                    newBoard.insert(n, player);
                    boards.add(newBoard);
                }
            }
        }
    }
    
    /**
     * Searches for the maximum value among the children(moves).
     * @a int initial smallest value.
     * @b int initial biggest value.
     */
    public int max(int a, int b)
    {
        //System.out.println("Depth in max: " + this.depth);
        if (this.depth < 0)
            return 0;
        if (brd.win())
            return -1;
        else 
        {
            int newa = a;
            best = boards.get(0).getMove();
            //System.out.println("/-----------------My board is this-----------------\\");
            //System.out.print(brd);
            //System.out.println("\\----------Now I'm looking at my children----------/");
            for (Iterator<Board> i = boards.iterator(); i.hasNext();)
            {
                Board board = i.next();
                MinMax node = new MinMax(board, depth - 1, !this.player);
                int n ;
                n = node.min(newa, b);
                //System.out.println("My min was "+n);
                //System.out.println(player);
                //System.out.println("Depth: " + this.depth);
                //System.out.println(n);
                //System.out.println("/-----------I'm a leaf. My board is this-----------\\");
                //System.out.print(board);
                //System.out.println("\\--------------------------------------------------/");
                if (newa < n)
                {
                    //System.out.println("!!!!!!!!!!!!!!!!!!!!!newa is now "+n+"!!!!!!!!!!!!!!!!!!!!!");
                    newa = n;
                    best = board.getMove();
                }
                if (b <= newa)
                    return newa;
            }
            //System.out.println("######################My value is " + newa + "######################");
            return newa;
        }
    }
    
    /**
     * Searches for the minimum value among the children(moves).
     * @a int initial smallest value.
     * @b int initial biggest value.
     */
    public int min(int a, int b)
    {
        //System.out.println("Depth in min: " + this.depth);
        if (this.depth < 0)
            return 0;
        if (brd.win())
            return 1;
        else 
        {
            int newb = b;
            best = boards.get(0).getMove();
            //System.out.println("/-----------------My board is this-----------------\\");
            //System.out.print(brd);
            //System.out.println("\\----------Now I'm looking at my children----------/");
            for (Iterator<Board> i = boards.iterator(); i.hasNext();)
            {
                Board board = i.next();
                MinMax node = new MinMax(board, depth - 1, !this.player);
                int n;
                n = node.max(a, newb); 
                //System.out.println("My max was "+n);
                
                if (n < newb)
                {
                    //System.out.println("!!!!!!!!!!!!!!!!!!!!!newb is now "+n+"!!!!!!!!!!!!!!!!!!!!!");
                    newb = n;
                    best = board.getMove();
                }
                if (newb <= a)
                    return newb;
            }
            //System.out.println("######################My value is " + newb + "######################");
            return newb;
        }
    }
    
    public int getBest()
    {
        return best;
    }
}
