
/**
 * MinMax represents a minmax ai algorithm, which is implemented as a game tree. 
 * The constructor creates a node that represents the board position. When the max method is called on that node, 
 * it looks at all its children which are legal moves from that position and calls the min method on the the children of the children.
 * Each position returns a max value which is favorable for the player and in this case it is 1.
 * The min value that is returned it is favorable for the opposite player and in this case it is - 1.
 * Finally 0 is the value that is returned if the move is not from a favorable value for any of the players.
 */
package Dajana;

import java.util.Iterator;
import java.util.ArrayList;

public class MinMax
{
    private Board brd, newBoard;
    private int depth, best;
    private Boolean player;
    private ArrayList<Board> boards;
    
    /**
     * Constructs nodes for each ligal move made on the current board.
     * @param board is the current Board that the players play on.
     * @player is a the color of the player making the move.
     */
    public MinMax(Board board, int depth, Boolean player)
    {
        brd = board;
        this.depth = depth;
        this.player = player;
        boards = new ArrayList<Board>();
        //System.out.println("Constractor Depth: " + this.depth);
        if (brd.win()) 
        {
            //System.out.println("==================Game Over");
        }
        else 
        {
            for (int n = 0; n < brd.getColumnSize(); n++) 
            {
                //System.out.print("Old board: \n" + brd);
                newBoard = new Board(brd);
                //System.out.print("New board: \n" + newBoard);
                if(newBoard.isValid(n))
                {
                    newBoard.insert(n, player);
                    boards.add(newBoard);
                }
            }
        }
    }
    
    /**
     * Searches for the maximum value among the children(moves).
     * @a int initial smallest value.
     * @b int initial biggest value.
     */
    public int max(int a, int b)
    {
        //System.out.println("Depth in max: " + this.depth);
        if (this.depth < 0)
            return 0;
        if (brd.win())
            return -1;
        else 
        {
            int newa = a;
            best = boards.get(0).getMove();
            //System.out.println("/-----------------My board is this-----------------\\");
            //System.out.print(brd);
            //System.out.println("\\----------Now I'm looking at my children----------/");
            for (Iterator<Board> i = boards.iterator(); i.hasNext();)
            {
                Board board = i.next();
                MinMax node = new MinMax(board, depth - 1, !this.player);
                int n ;
                n = node.min(newa, b);
                //System.out.println("My min was "+n);
                //System.out.println(player);
                //System.out.println("Depth: " + this.depth);
                //System.out.println(n);
                //System.out.println("/-----------I'm a leaf. My board is this-----------\\");
                //System.out.print(board);
                //System.out.println("\\--------------------------------------------------/");
                if (newa < n)
                {
                    //System.out.println("!!!!!!!!!!!!!!!!!!!!!newa is now "+n+"!!!!!!!!!!!!!!!!!!!!!");
                    newa = n;
                    best = board.getMove();
                }
                if (b <= newa)
                    return newa;
            }
            //System.out.println("######################My value is " + newa + "######################");
            return newa;
        }
    }
    
    /**
     * Searches for the minimum value among the children(moves).
     * @a int initial smallest value.
     * @b int initial biggest value.
     */
    public int min(int a, int b)
    {
        //System.out.println("Depth in min: " + this.depth);
        if (this.depth < 0)
            return 0;
        if (brd.win())
            return 1;
        else 
        {
            int newb = b;
            best = boards.get(0).getMove();
            //System.out.println("/-----------------My board is this-----------------\\");
            //System.out.print(brd);
            //System.out.println("\\----------Now I'm looking at my children----------/");
            for (Iterator<Board> i = boards.iterator(); i.hasNext();)
            {
                Board board = i.next();
                MinMax node = new MinMax(board, depth - 1, !this.player);
                int n;
                n = node.max(a, newb); 
                //System.out.println("My max was "+n);
                
                if (n < newb)
                {
                    //System.out.println("!!!!!!!!!!!!!!!!!!!!!newb is now "+n+"!!!!!!!!!!!!!!!!!!!!!");
                    newb = n;
                    best = board.getMove();
                }
                if (newb <= a)
                    return newb;
            }
            //System.out.println("######################My value is " + newb + "######################");
            return newb;
        }
    }
    
    public int getBest()
    {
        return best;
    }
}
