package EsbenBirkebaekLarsen;

import Player.Player;

import java.util.ArrayList;

public class EsbenBirkebaekLarsen implements Player
{
	
	public int move()
	{
		
		class BestMove
		{
			ArrayList<ComputerBoard> boards = new ArrayList<ComputerBoard>();
			ComputerBoard board;
			int depth;
			boolean gameWon;
			int bestMove;
			boolean redsTurn;
			
			BestMove(ComputerBoard b, int depth, boolean redsTurn)
			{
				this.gameWon = false;
				this.board = b;
				this.depth = depth;
				this.redsTurn = redsTurn;
				if(board.gameIsOver())
				{
					this.gameWon = true;
				}
				else
				{
					for(int i = 0; i <= POSSIBLE_MOVES; i++)
						{
							if(board.spaceInColumn(i))
								boards.add(new ComputerBoard(board.getBoardSituation(), i, redsTurn));
						}
				}
			}
			
			int max() 
			{
				if(gameWon) 	//stops if game is finished. Best possible outcome is assigned (100)
					return -100; 
				else if(this.depth == 0) // if depth = 0, evaluate and stop the recursion            
				{
					int result = board.evaluateWinnerChance() * -1;
					return result;
				}
				else 									
				{
					bestMove = boards.get(0).getLastMove();
					int result = -101;
					// save the best situation
					for(ComputerBoard e: boards)
					{
						int minValue = new BestMove(e, depth -1, !redsTurn).min();
						if(minValue > result)
						{
							result = minValue;
							this.bestMove = e.getLastMove();
						}
						if(result >= 100) return result;
					}
					return result;
				}
			}
			
			int min()
			{
				if(gameWon)
					return 100;
				else if(this.depth == 0)
				{
					int result = board.evaluateWinnerChance();
					// compute the quality of the board 
					return result;
				}
				else
				{
					bestMove = boards.get(0).getLastMove();
					int result = 101;
					for(ComputerBoard e: boards)
					{
						int minValue = new BestMove(e, depth - 1, !redsTurn).max();
						if (minValue < result)
						{
							result = minValue;
							this.bestMove = e.getLastMove();
						}
						if(result <= -100) return result;
					}
					return result;
				}
			}
		}
		
		int depth = 6 ; // will take about 8 second, we have max 18 second to compute 5 steps ahead
		boolean isRed = color;
		BestMove gen = new BestMove(myBoard, depth, isRed);
		gen.max();
		int move = gen.bestMove;
		myBoard.inform(move);
		return move;	
		
	}
	
	public void inform(int i)
	{
		myBoard.inform(i);
	}	
	
	public EsbenBirkebaekLarsen()
	{
		myBoard = new ComputerBoard();
	}
	
	public void init(Boolean color)
	{
		this.color = color;
	}
	
	public String name()
	{
		return name;
	}
	
	private String name = "Esben";
	
	private boolean color;
	
	private ComputerBoard myBoard;
	
	private final int POSSIBLE_MOVES = 6; 
}


class ComputerBoard
{
	 // Constructs a new empty board.
	 
	public ComputerBoard()
	{
		gameWon = false;
		redsTurn = true;
		board = new int[BOARD_WIDTH][BOARD_HEIGHT];
			
		for(int i = 0; i < BOARD_WIDTH; i++)	
			for(int j = 0; j < BOARD_HEIGHT; j++)
				board[i][j] = 0;
	}

	/**
	 * Initializes a board with some tokens already played
	 * @param boardSituation
	 */
	public ComputerBoard(int[][] boardSituation, int nextMove, boolean redsTurn)
	{
		gameWon = false;
		this.redsTurn = redsTurn;
		this.board = boardSituation;
		informAndCheckIfWinner(nextMove);
	}
	
	public int[][] getBoardSituation()
	{
		int[][] situation = new int[BOARD_WIDTH][BOARD_HEIGHT];
		for(int i = 0; i < BOARD_WIDTH; i++)	
			for(int j = 0; j < BOARD_HEIGHT; j++)
				situation[i][j] = board[i][j];
		return situation;
	}
	
	/**
	 * It is a precondition for this method that the move is a legal move.
	 * The method is used for informing the board about which move has just been made.  
	 * The method returns true, if the move wins the game.
	 */
	public boolean informAndCheckIfWinner(int columnNumber)
	{
		lastMove = columnNumber;
		int color; 
		
		if (redsTurn)
			{
				color = 1; // red
				redsTurn = false;
			}
		else
			{
				color = -1; // yellow
				redsTurn = true;
			}
		
	
		// Iterates through the column from bottom to top and inserts a token when encountering an empty square.
			
		for(int j = BOARD_HEIGHT -1; j >= 0; j--)
		{	
			if(board[columnNumber][j] == 0)
			{
				board[columnNumber][j] = color; // Inserts a "color" token.
				rowNumber = j;
				gameWon = lookForWinner(columnNumber, rowNumber, color, 1);
				break;
			}
		}
		return gameWon; 
	}
		
	
	/**
	 * Informs the board that a move has been made in the column, and remembers that move as the last.
	 * @param columnNumber
	 */
	public void inform(int columnNumber)
	{
		lastMove = columnNumber;
		int color; 
		
		if (redsTurn)
			{
				color = 1; // red
				redsTurn = false;
			}
		else
			{
				color = -1; // yellow
				redsTurn = true;
			}
		
		// Iterates through the column from bottom to top and inserts a token when encountering an empty square.
			
		for(int j = BOARD_HEIGHT -1; j >= 0; j--)
		{
					
			if(board[columnNumber][j] == 0)
			{
				board[columnNumber][j] = color; // Inserts a "color" token.
				break;
			}
		}
		
	}
	
	/**
	 * Checks if the position at (i, j) is legal
	 */
	public boolean positionIsLegal(int i, int j)
	{
		if (i < 0 || i >= BOARD_WIDTH || j < 0 || j >= BOARD_HEIGHT) return false;
		else return true;
	}
		
	/**
	 * This method check if the token at column (i,j) is part of a winning combination - four in a row. The method return
	 * true if this is the case
	 */
	
	public boolean lookForWinner(int i, int j, int color, int counter)
	{
		boolean result = false;
			
		// The 1. token does not count itself.
		boolean bothDirectionsChecked = false; // The recursion goes "both ways" along a direction, initially it has not 
											  // checked both direction
		if (directionCheck(i,j, 1, 0, color, counter, bothDirectionsChecked)) result = true;
		else if	(directionCheck(i, j, 0, 1, color, counter, bothDirectionsChecked)) result = true;
		else if	(directionCheck(i,j, 1, -1, color, counter, bothDirectionsChecked)) result = true;
		else if	(directionCheck(i, j, 1, 1, color, counter, bothDirectionsChecked)) result = true;
		return result;
	}
		
	/**
	 * This method recursively checks if a token is in a winning combination in direction dX/dY
	 * @param i x-coordinate of the token
	 * @param j y-coordinate of the token
	 * @param dX When checking the next token the position changes dX on the x-axis
	 * @param dY When checking the next token the position changes dY on the y-axis
	 * @param color the color of the token
	 * @param counter 
	 * @param bothDirectionsChecked
	 * @return
	 */
	public boolean directionCheck(int i, int j, int dX, int dY, int color, int counter, boolean bothDirectionsChecked)
	{
		boolean result = false;
			
		// (checks if the move is not legal or the token does not have the right color) and both directions have been checked
		// we return false;
		if((!positionIsLegal(i +dX, j + dY) || !(board[i + dX][j + dY] == color)) && bothDirectionsChecked) 
		{
			result = false; 
		}
		// (checks if the move is not legal or the token does not have the right color) and both directions have NOT been checked
		// the counter is reset, and a new recursion is started in the OPPOSITE direction.
		else if ((!positionIsLegal(i +dX, j + dY) || !(board[i + dX][j + dY] == color)) && !bothDirectionsChecked) 
		{
			counter = 1;
			bothDirectionsChecked = true;
			dX = -dX;
			dY = -dY;
			result = directionCheck(i, j, dX, dY, color, counter, bothDirectionsChecked);
		}
		// If the counter reaches four we have a winning combination and return true. If not we check the next token in
		// the dX/dY direction.
		else
		{
			counter++;
			if(counter == 4) 
				{
					result = true;
				}
			else
				result = directionCheck(i + dX, j + dY, dX, dY, color, counter, bothDirectionsChecked);
		}
		return result;
	}
		
	/**
	 * Returns the next free position in the column i, and adds a token to that position. If the column is full
	 * -1 is returned. 
	 * @param columnNumber the column to drop the token into
	 * @return result The first free row number
	 */
	public int informAndReturnRow(int columnNumber)
	{
		int rowNumber = -1; // If the column is full the method return -1;
		if(redsTurn) 
		{
			// Iterates through the column from bottom to top and inserts a red token when encountering an empty square.
			for(int j = BOARD_HEIGHT -1; j >= 0; j--)
			{
					
				if(board[columnNumber][j] == 0)
				{
					board[columnNumber][j] = 1; 
					rowNumber = j; 
					System.out.println();
					gameWon = lookForWinner(columnNumber, rowNumber, 1, 1);
					break;
				}
			}
			redsTurn = false;			
		}
		else 
		{
			for(int j = BOARD_HEIGHT -1; j >= 0; j--)
			{
				if(board[columnNumber][j] == 0)
				{
					board[columnNumber][j] = -1;
					rowNumber = j;
					gameWon = lookForWinner(columnNumber, rowNumber, -1, 1);
					break;
				}
			}
			redsTurn = true;
		}
		return rowNumber;
	}
	
	
	/**
	 * Evaluates the winner chance of the current board for red if color is true else for yellow.
	 */
	public int evaluateWinnerChance()
	{
		int color;
		if(redsTurn) 
			color = 1;
		else
			color = -1;
		
		boolean good = lookForWinner(lastMove, rowNumber, color, 2);
		if(good)
			return 50;
		else
			return 0;
	}		
	
	public boolean gameIsOver()
	{
		return gameWon;
	}
	
	public int getLastMove()
	{
		return lastMove;
	}
	
	public void printBoard()
	{
		for(int i = 0; i < 6; i++)
		{
			System.out.print("\n");
			for(int j = 0; j < 7; j++)
				System.out.print(board[j][i]);
		}
		System.out.println();
	}
	
	public boolean spaceInColumn(int i) 
	{
		if(board[i][0] == 0) 
			return true;
		else 
			return false;
	}
	private boolean gameWon;
	public boolean redsTurn;
	private int[][] board;
	private final static int BOARD_WIDTH = 7;
	private final static int BOARD_HEIGHT = 6;
	private int rowNumber;
	private int lastMove;
}





