package Garry;
import Player.Player;

class MinMax {
	static final int[] possibleMoves = new int[]{3,4,2,5,1,6,0};  // start in the centre
	Board board;
    int depth;
    int won;

MinMax (Board board, int depth, int move) {
   	this.board = board;
   	this.depth = depth;
   	if (board.win()) this.won = 10 - board.getHeight(move);  // weight it so lower on the board is better
   	else this.won = 0;
}
    
int max (int a, int b) {
   	if (this.depth <= 0) return 0;
   	if (this.won > 0) return -this.won;

   	int newa = a;
    board.bestMove = -1;
    for (int i = 0; i < Board.MAXCOLUMN; i++)                    // all moves
    	if (board.getHeight(possibleMoves[i]) < Board.MAXROW) {  // in unfilled columns
    		if (board.bestMove == -1) board.bestMove = possibleMoves[i];  // precautionary
    		Board copy = (Board) board.clone();
    		copy.dropToken(possibleMoves[i]);
       		int n = new MinMax (copy, this.depth-1,possibleMoves[i]).min (a, b);
    		if (newa < n) {newa = n; board.bestMove = possibleMoves[i];}
    		if (b <= newa) return newa;
    	}
    return newa;
}

int min (int a, int b) {
   	if (this.depth <= 0) return 0; 
  	if (this.won > 0) return this.won;

  	int newb = b;
	board.bestMove = -1;
   	for (int i = 0; i < Board.MAXCOLUMN; i++)                    // all moves
   		if (board.getHeight(possibleMoves[i]) < Board.MAXROW) {  // in unfilled columns
   			if (board.bestMove == -1) board.bestMove = possibleMoves[i];  // precautionary
   			Board copy = (Board) board.clone();
   			copy.dropToken(possibleMoves[i]);
   			int n = new MinMax (copy,  this.depth-1,possibleMoves[i]).max (a, b);
  			if (n < newb) newb = n; board.bestMove = possibleMoves[i];
   			if (newb <= a) return newb;
   		}
		return newb;
}

}

/* Connect 4 */
/* My wonderful Connect4 Player implementation */

public class Garry implements Player {
	Board b;
     
	public void init (Boolean color) { b = new Board();}
    public String name () { return "Garry"; }

   
    public int move (){ 
       	int move = -1;
        
    	if ( b.getNumTokens() <= 4 ) {     // Use a Playbook initially
       		switch( b.getNumTokens()) { // switch on tokens already played gt(c,r)
       	 	case 0: move = 3;
       	 	break;
       	 	case 1: if (b.getTokenColour(3,0) == 'R'); move = 2; // strong but beatable
       	   	break;
      	 	case 2: if ((b.getTokenColour(3,0) == 'R' && b.getTokenColour(2,0) == 'Y') ||
      	 	 (b.getTokenColour(3,0) == 'R' && b.getTokenColour(1,0) == 'Y')) move = 5; // winner white
    	 	else  if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(3,1) == 'Y') move = 2;
      	 	break;
      	 	case 3: if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(1,0) == 'Y' && b.getTokenColour(5,0) == 'R') move = 4;
     	 	else if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(2,0) == 'Y' && b.getTokenColour(2,1) == 'R') move = 2;
     	 	else if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(3,1) == 'Y' && b.getTokenColour(2,0) == 'R') move = 4;
      	   	break;
      	 	case 4: if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(2,0) == 'Y' && b.getTokenColour(5,0) == 'R' && b.getTokenColour(6,0) == 'Y') move = 5;
    	 	else  if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(1,0) == 'Y' && b.getTokenColour(5,0) == 'R' && b.getTokenColour(2,0) == 'Y') move = 4;
    	 	else  if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(1,0) == 'Y' && b.getTokenColour(5,0) == 'R' && b.getTokenColour(6,0) == 'Y') move = 4;
    	 	else  if (b.getTokenColour(3,0) == 'R' && b.getTokenColour(1,0) == 'Y' && b.getTokenColour(5,0) == 'R' && b.getTokenColour(4,0) == 'Y') move = 5;
      	   	break;
    	 	default:
    	 		move = -1;
       		}
   	}
    	if (move == -1) {                             // use minimax
    		int fullCols = 0;
    		for (int i = 0; i < Board.MAXCOLUMN; i++ )
    			if (b.getHeight(i) == Board.MAXROW)
    				fullCols++;
    		int levels = 8+fullCols;
    		MinMax start = new MinMax(b,levels,-1);
       		start.max(-1000, 1000);
       		move = b.bestMove;		
       	}

    	b.dropToken(move);
    	return(move);
    }

    public void inform (int i) {
    	b.dropToken(i);
    }
}


class Board implements Cloneable {

// The algorithms are guaranteed NOT to work if MAXCOLUMN and/or MAXROW are
// modified. These constants are only used to better document the code.

// The board is a bitmap, bits set to true to show the position of all tokens of
// colour "boardColour", the rest of the board is false.
// The array height indicates how many tokens the column contains, of both
// colours. The position of the tokens of !boardColour are therefore easy to
// generate.
//
// NB .. unused cells at top of column makes addressing faster. A cell is then
// addressed as column*8+row (otherwise it would be column*6+row). As *8 is just
// a left bit-shift << 3, it saves doing a multiplication, which is expensive.
// Optimisations like these are usually noted where they occur. However, there
// may be other dependencies on a 6 by 7 board that make constants like MAXCOLUMN
// and MAXROW unchangeable.  

//| .| .| .| .| .| .| .| .|		The unused cells ... may be used to store
//| .| .| .| .| .| .| .| .|		the height info. Then the state of a game 
//| 5|11|17|23|29|35|41| .|		could be packed into 64 bits.
//| 4|10|16|22|28|34|40| .| 
//| 3| 9|15|21|27|33|39| .|
//| 2| 8|14|20|26|32|38| .|
//| 1| 7|13|19|25|31|37| .|
//| 0| 6|12|18|24|30|36|42|

public static final int MAXCOLUMN = 7;          // board width
public static final int MAXROW = 6;             // board height
private long red;                               // 64bit bitmap of red coloured tokens
private long yellow;                            // 64bit bitmap of yellow coloured tokens
private byte[] height = new byte[MAXCOLUMN+1];  // number of tokens in each column, 8th holds total number of tokens played
public int bestMove;


/**
 * Constructor: sets up an empty bit board.
 * @param colour is the colour of the token that is represented by true.
 */
public Board(){
	init();
}

/**
 * Override clone() to clone a Board including its state.
 */
public Object clone(){
	try {
		Board clone = (Board)super.clone();
		clone.height = (byte[]) height.clone();	// clone the array !
		return clone;
	}
	catch (CloneNotSupportedException a){return null;} // Java stupidity
}

/**
 * Initialise the state of the board.
 * @param colour is the colour of the token that is represented by true.
 */
public void init (){
	red = yellow = 0;                           // set all positions to ~boardColour
	height = longToByteArray(0);                // set all columns to zero, including total
	// height[7] == 0 means nextToken to be played is red
}
	
/**
 * Low level method to drop the next token in a column. No checking of parameters!
 * Has the side effect of adding a bit for the token to the bit-board and increasing column height.
 * 
 * @invariant height[c] <= MAXROW for all c = 0..MAXCOLUMN-1
 * @precondition  parameter "column" in range [0 .. MAXCOLUMN-1]
 * @precondition  height["column"] < MAXROW
 * @param column is the column that a token is to be dropped into.
 */
public void dropToken(int column){

	long addr = 1;		
	// addr <<= column*(MAXROW + (2 unused cells)=8) + height[column]++;
	addr <<= (column << 3) + height[column];    // set the bit
	
	if (((height[MAXCOLUMN] & 0x01) == 0x00 ))  // RED turn to play
		red |= addr;
	else
		yellow |= addr;
	height[column]++;                           // increase column height
	height[MAXCOLUMN]++;                        // numTokens++; update number of tokens played
}

/**
 * Get the colour of the last token played.
 * @return true if RED false if YELLOW
 */
public boolean lastToken(){     // Uses a bit mask to find out if total played is odd or even
	return ((height[MAXCOLUMN] & 0x01) == 0x01 );  // evaluates to true if last token was RED
}

/**
 * Determines if the last token added wins the game.
 * @return true if last token makes 4 in a row
 */
public boolean win4(){
	long slide;     // board is slid around, counting how many are in a row.
	long win;       // result of the test for 4 in a row in each direction

	// The general algorithm is:
	// Test in vertical direction (column or Y axis)
	//    - slide the new board along 1 (1 bit to the left = 1 in the Y direction)
	//    - match each token against its predecessor (3 must match)
	//    - slide 2 more, total of 3 and check that token 1 matches token 4
	//      and hence 4 in a row
	// Repeat, for the other directions 
	//      (slide by 7,8,9 bits to slide 1 in the \,-,/ direction respectively)
	// Note: all directions are tested, even if 1 is successful. Failure is more frequent than
	//		success and the tests require to exit early will likely interfere with the 
	//		pipelining in most processors.

	long board = ((height[MAXCOLUMN] & 0x01) == 0x01 ) ? red : yellow; // check board that was played last

	// check VERTICAL
	slide = board & board << 1;          // slide up 1, match adjacent tokens
	win = slide & slide << 2;            // check token 1 and 4 match, win!=0 if 4 in a row
	// check HORIZONTAL
  	slide = board & board << 8;          // slide across 1, (8 bits left) and match
	win |= slide & slide << 16;          // check token 1 and 4 match
	// check SLASH diagonal
	slide = board & board << 9;          // slide across 1, up 1 (8+1 bits left) and match
	win |= slide & slide << 18;          // check token 1 and 4 match
	// check BACKSLASH diagonal
  	slide = board & board << 7;          // slide across 1, down 1 (8-1 bits left) and match
	win |= slide & slide << 14;          // check token 1 and 4 match, add to the win

	return win != 0;                     // if win!=0 then (4 in a row) in at least 1 direction
}
public boolean win3(){
	long slide;     // board is slid around, counting how many are in a row.
	long win;       // result of the test for 3 in a row in each direction

	// The general algorithm is:
	// Test in vertical direction (column or Y axis)
	//    - slide the new board along 1 (1 bit to the left = 1 in the Y direction)
	//    - match each token against its predecessor (3 must match)
	//    - slide 1 more, total of 2 and check that token 1 matches token 3
	//      and hence 3 in a row
	// Repeat, for the other directions 
	//      (slide by 7,8,9 bits to slide 1 in the \,-,/ direction respectively)
	// Note: all directions are tested, even if 1 is successful. Failure is more frequent than
	//		success and the tests require to exit early will likely interfere with the 
	//		pipelining in most processors.

	long board = ((height[MAXCOLUMN] & 0x01) == 0x01 ) ? red : yellow; // check board that was played last

	// check VERTICAL
	slide = board & board << 1;          // slide up 1, match adjacent tokens
	win = slide & slide << 1;            // check token 1 and 3 match, win!=0 if 4 in a row
	// check HORIZONTAL
  	slide = board & board << 8;          // slide across 1, (8 bits left) and match
	win |= slide & slide <<  8;          // check token 1 and 3 match
	// check SLASH diagonal
	slide = board & board << 9;          // slide across 1, up 1 (8+1 bits left) and match
	win |= slide & slide <<  9;          // check token 1 and 3 match
	// check BACKSLASH diagonal
  	slide = board & board << 7;          // slide across 1, down 1 (8-1 bits left) and match
	win |= slide & slide <<  7;          // check token 1 and 3 match, add to the win

	return win != 0;                     // if win!=0 then (3 in a row) in at least 1 direction
}
public boolean win2(){
	long win;       // result of the test for 2 in a row in each direction

	// The general algorithm is:
	// Test in vertical direction (column or Y axis)
	//    - slide the new board along 1 (1 bit to the left = 1 in the Y direction)
	//    - match each token against its predecessor (2 must match)
	// Repeat, for the other directions 
	//      (slide by 7,8,9 bits to slide 1 in the \,-,/ direction respectively)
	// Note: all directions are tested, even if 1 is successful. Failure is more frequent than
	//		success and the tests require to exit early will likely interfere with the 
	//		pipelining in most processors.

	long board = ((height[MAXCOLUMN] & 0x01) == 0x01 ) ? red : yellow; // check board that was played last

	// check VERTICAL
	win = board & board << 1;           // slide up 1, match adjacent tokens
	// check HORIZONTAL
	win |= board & board << 8;          // slide across 1, (8 bits left) and match
	// check SLASH diagonal
	win |= board & board << 9;          // slide across 1, up 1 (8+1 bits left) and match
	// check BACKSLASH diagonal
	win |= board & board << 7;          // slide across 1, down 1 (8-1 bits left) and match

	return win != 0;                    // if win!=0 then (4 in a row) in at least 1 direction
}


/**
 * Determine if the game is a draw.
 * @return true is a draw
 */
public boolean tie(){ return height[MAXCOLUMN] >= 42; }


/**
 * Get the colour of a token from the board as a String in ["R","Y"," "].
 * @precondition  "column" in range [0 .. MAXCOLUMN-1]
 * @precondition  "row" in range [0 .. MAXROW-1]
 * @param column of an array[column][row] containing tokens.
 * @param row of an array[column][row] containing tokens.
 * @return the token at the specified position.
 */
public char getTokenColour(int column, int row){
	// if row > height of column return " "
	// else if contents of (address = column*8 + row) is true return "R"
	// else return "Y".
	long addr = 1;
	if ( height[column] <= row )        // position has not been played
		return ' ';
	return ((red & (addr << (column << 3) + row)) != 0) ? 'R' : 'Y'; // position is set or not
}

/**
 * A textual representation of the board. 
 */
public String toString(){
   	String s = "\n";
	for (int r = MAXROW; --r >= 0; s += "|\n")
		for (int c = 0; c < MAXCOLUMN; c++)
			s += "|" + getTokenColour(c,r);
	return s;
}
	
/**
 * Get height of a column.
 * @precondition column in range [0 .. MAXCOLUMN-1]
 * @param "column" of the board.
 * @return height[column-1]
 */
public int getHeight(int column){
	return height[column];
}

/**
 * Get number of tokens played so far.
 * @return numTokens
 */
public int getNumTokens(){return height[MAXCOLUMN];}

public static final byte[] longToByteArray(long value) {
    return new byte[] {
            (byte)(value >>> 56),
            (byte)(value >>> 48),
            (byte)(value >>> 40),
            (byte)(value >>> 32),
            (byte)(value >>> 24),
            (byte)(value >>> 16),
            (byte)(value >>> 8),
            (byte)value};
}


//The inverse of this, byte array to int, would be:
//The masks are needed here because when the byte is widened to an int sign extension may add lots of bits.

public static final long byteArrayToLong(byte [] b) {
    return (b[0] << 56)
    + ((b[1] & 0xFF) << 48)
    + ((b[2] & 0xFF) << 40)
    + ((b[3] & 0xFF) << 32)
    + ((b[4] & 0xFF) << 24)
    + ((b[5] & 0xFF) << 16)
    + ((b[6] & 0xFF) << 8)
    + (b[7] & 0xFF);
}
public void print(){ System.out.println(this.toString()); }
public Boolean isValid (int m) {
	if (m >= 0 && m < 7) {return true;}
	else {return false;}
    }
public void insert (int m, Boolean player)  {
		dropToken(m);

}
public Boolean win(Boolean player){
	long slide;     // board is slid around, counting how many are in a row.
	long win;       // result of the test for 4 in a row in each direction

	long board = (player) ? red : yellow; // check board according to player

	// check VERTICAL
	slide = board & board << 1;          // slide up 1, match adjacent tokens
	win = slide & slide << 2;            // check token 1 and 4 match, win!=0 if 4 in a row
	// check HORIZONTAL
  	slide = board & board << 8;          // slide across 1, (8 bits left) and match
	win |= slide & slide << 16;          // check token 1 and 4 match
	// check SLASH diagonal
	slide = board & board << 9;          // slide across 1, up 1 (8+1 bits left) and match
	win |= slide & slide << 18;          // check token 1 and 4 match
	// check BACKSLASH diagonal
  	slide = board & board << 7;          // slide across 1, down 1 (8-1 bits left) and match
	win |= slide & slide << 14;          // check token 1 and 4 match, add to the win

	return win != 0;                     // if win!=0 then (4 in a row) in at least 1 direction

}
}
