package JanusBrunkOlsen;

import Player.Player;

import javax.swing.JOptionPane;

public class JanusBrunkOlsen implements Player {

	/**
	 * JanusBrunkOlsen - sets the name of a machine player and inits
	 * a new board, could in time be used to implement an interface
	 * where the game had highscore..etc. (can be omitted from init)
	 * @return void
	 */
	public JanusBrunkOlsen() { // player is initialized with a name
		name = "Janus";
	}

	/**
	 * INFORM - is used to let the Connect4Viewer feed the data
	 * from a move back to the -opponent- of the player that made
	 * the current move
	 * @return void
	 */
	public void inform(int i) { // player is informed about the move of the opponent (by GUI or referee)
		maxHeight[i]++;
		if (playerColor == 1) {
			xBoard.set(i, 2);
		}
		else {
			xBoard.set(i, 1);
		}
	}

	/**
	 * PRINTCURRENTBOARD - prints current board to the console for debugging
	 * @return void
	 */
	public void printCurrentBoard(String temp){
		System.out.println("playerMachineRED_" + playerColor + "has so board:");
		System.out.println(temp);
		System.out.println("|   "+ xBoard.get(0)+"|   "+xBoard.get(1)+"   "+xBoard.get(2)+"|   "+xBoard.get(4)+"|   "+xBoard.get(4)+"|   "+xBoard.get(5)+"|   "+xBoard.get(6)+"|");
		System.out.println("|   "+ xBoard.get(7)+"|   "+xBoard.get(8)+"   "+xBoard.get(9)+"|   "+xBoard.get(10)+"|   "+xBoard.get(11)+"|   "+xBoard.get(12)+"|   "+xBoard.get(13)+"|");
		System.out.println("|   "+ xBoard.get(14)+"|   "+xBoard.get(15)+"   "+xBoard.get(16)+"|   "+xBoard.get(17)+"|   "+xBoard.get(18)+"|   "+xBoard.get(19)+"|   "+xBoard.get(20)+"|");
		System.out.println("|   "+ xBoard.get(21)+"|   "+xBoard.get(22)+"   "+xBoard.get(23)+"|   "+xBoard.get(24)+"|   "+xBoard.get(25)+"|   "+xBoard.get(26)+"|   "+xBoard.get(27)+"|");
		System.out.println("|   "+ xBoard.get(28)+"|   "+xBoard.get(29)+"   "+xBoard.get(30)+"|   "+xBoard.get(31)+"|   "+xBoard.get(32)+"|   "+xBoard.get(33)+"|   "+xBoard.get(34)+"|");
		System.out.println("|   "+ xBoard.get(35)+"|   "+xBoard.get(36)+"   "+xBoard.get(37)+"|   "+xBoard.get(38)+"|   "+xBoard.get(39)+"|   "+xBoard.get(40)+"|   "+xBoard.get(41)+"|");
	}

	/**
	 * INIT - initializes the player, true for red false for yellow,
	 * in this implementation this translates to 1 for red and 2
	 * for yellow
	 * @return void
	 */
	public void init(Boolean color) { // red is TRUE, yellow is FALSE
		xBoard = new Board();
		if (color.equals(true)) {
		playerColor = 1;
		opponentColor = 2;
		}
		else {opponentColor = 1; playerColor = 2;}
		maxHeight = new int[]{0,0,0,0,0,0,0};
	}

	/**
	 * MOVE - tells the player to move, initializes AI, updates own board
	 * and finally returns move to Connect4Viewer/referee
	 * @return 0 always
	 */
	public int move() { // player is asked to make a move (by GUI or referee)
		scoreAI = new int[]{0,0,0,0,0,0,0}; // scoreboard
		for (int i = 0; i < 7;i++){scoreAI[i] = 0;}

		for (int i = 0; i <= 6;i++) { // each column is awarded points based on matching rules
			if (maxHeight[i] < 6 && WinInNext(i)) 				{scoreAI[i] += 50;}
			if (maxHeight[i] < 6 && BlockInNext(i)) 			{scoreAI[i] += 20;}
			if (maxHeight[i] < 6 && Block3InNext(i)) 			{scoreAI[i] += 12;}
			if (maxHeight[i] < 6 && LooseInNextIfSet(i)) 		{scoreAI[i] -= 30;}
			if (maxHeight[i] < 6 && Connect3Next(i)) 			{scoreAI[i] += 11;}
			if (maxHeight[i] < 6 && Connect2HoriRightInNext(i)) {scoreAI[i] += 2;}
			if (maxHeight[i] < 6 && Connect2HoriLeftInNext(i)) 		{scoreAI[i] += 2;}
			if (maxHeight[i] < 6 && Connect2VerticalInNext(i)) 	{scoreAI[i] += 4;}
		}
		int bestscoreAI = ReturnBestScoreColumn(scoreAI);
		if (maxHeight[bestscoreAI] >= 5) {
			for (int i = 0; i <= 6;i++) {
				if (maxHeight[i] < 6){xBoard.set(i, playerColor);maxHeight[i]++;return i;}
			}
		}
		System.out.println("BESTSCOREAI at column: " + bestscoreAI);
		xBoard.set(bestscoreAI, playerColor);
		this.printCurrentBoard("testing, after set");
		maxHeight[bestscoreAI]++;
		return bestscoreAI;
	}

	/**
	 * RETURNBESTSCORECOLUMN - returns the column that the AI has
	 * rated highest
	 * @param array to evaluate
	 * @return column
	 */
	public int ReturnBestScoreColumn(int[] scoreAI) {
		int highest = scoreAI[0];
		int index = 0;
		for (int i = 0; i <= scoreAI.length-1; i++){
			System.out.println("scoreAI["+i+"]: " + scoreAI[i]);
			if (scoreAI[i] > highest){highest = scoreAI[i]; index = i;}
		}
		return index;
	}

	/**
	 * CONNECT2VERTICALINNEXT - returns a column that allows the
	 * ai to connect to two tokens vertically
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect2VerticalInNext(int iColumn) {
		int bottom = xBoard.getToken(iColumn);
		xBoard.set(iColumn, playerColor);
		if (bottom == xBoard.getToken(iColumn)) {
			xBoard.unset();
			return true;
		}
		xBoard.unset();
		return false;
	}

	/**
	 * CONNECT2HORIZONTALLEFTINNEXT - returns a column that allows the
	 * ai to connect to two tokens horizontally (to the left of current
	 * token)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect2HoriLeftInNext(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (iColumn >= 1 && iColumn <= 6) {
			if (xBoard.getToken(iColumn) == xBoard.getToken(iColumn-1)) {
				xBoard.unset();
				return true;
			}
		}
		xBoard.unset();
		return false;
	}

	/**
	 * CONNECT2HORIZONTALRIGHTINNEXT - returns a column that allows the
	 * ai to connect to two tokens horizontally (to the right of current
	 * token)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect2HoriRightInNext(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (iColumn <= 5) {
			if (xBoard.getToken(iColumn) == xBoard.getToken(iColumn+1)) {
				xBoard.unset();
				return true;
			}
		}
		xBoard.unset();
		return false;
	}

	/**
	 * CONNECT3NEXT - returns a column that allows the
	 * ai to connect to two tokens (making it 3 connected tokens..)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect3Next(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (xBoard.win3(playerColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * BLOCK3NEXT - returns a column that allows the
	 * ai to block to two tokens (from becoming 3)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Block3InNext(int iColumn) {
		xBoard.set(iColumn, opponentColor);
		if (xBoard.win3(opponentColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * BLOCKINNEXT - returns a column that allows the
	 * ai to block a opponent win
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean BlockInNext(int iColumn) {
		xBoard.set(iColumn, opponentColor);
		if (xBoard.win(opponentColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * BLOCKINNEXT - returns a column that allows the
	 * ai to win
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean WinInNext(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (xBoard.win(playerColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * LOOSEINNEXTIFSET - returns a column that allows the
	 * ai to know where -not- to put a token (because it
	 * would cause it to loose)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean LooseInNextIfSet(int iColumn) {
		xBoard.set(iColumn, playerColor);
		xBoard.set(iColumn, opponentColor);
		if (xBoard.win(opponentColor) == 1) {xBoard.unset();xBoard.unset();return true;}
		else {xBoard.unset();xBoard.unset();return false;}
	}

	/**
	 * ISDRAWNCHECK - checks if there is a win on the board associated
	 * to the player, relies on "isgameDrawn" function in board class
	 * @return void
	 */
	public void isDrawnCheck(){
		if (xBoard.isgameDrawn()) {
			JOptionPane.showMessageDialog(null, "Game is over, no winner.",
			"Game drawn", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * WIN - checks if there is a win on the board associated
	 * to the player, relies on "win" function in board class
	 * @return 1 if win, 0 if not
	 */
	public int win() { // player checks if game be won
		int win=0;
		win = xBoard.win(playerColor);
		return win;
	}

	/**
	 * RESET - makes a new board for the player
	 * @return void
	 */
	public void reset() { // return the name of the player
		xBoard = new Board();
	}

	/**
	 * NAME -  returns the name of the player
	 * @return name, string
	 */
	public String name() { // return the name of the player
		return name;
	}

	// INSTANCE FIELD
	String name;
	Board xBoard;
	int playerColor;
	int opponentColor;
	int win = 0;
	Boolean firstround = true;
	int[] scoreAI;
	int[] maxHeight;
}

class Board {

	/**
	 * Board - Constructor for connect4 playing field
	 * @return void
	 */
	public Board() {
		board = new int[42];
		for (int i = 0; i < 42;i++) {board[i] = 0;}
	}

	/**
	 * GET - Gets token, can return from anywhere on board;
	 * used by printCurrentBoard in Player(s) for debugging
	 * @param int location of token to get (0-41)
	 * @return void
	 */
	public int get(int x) {
		return board[x];
	}

	/**
	 * SET - Sets token on board
	 * @return void
	 */
	public void set(int tempPosition, int tempColor) {
		lastMove = tempPosition;
		drawCounter++;
		int temp = tempPosition +(7*5); // position at the bottom of a column
		//if(board[tempPosition] != 0) {return 0;}
		for (int j = 5;j >= 0; j--) {
			if(board[temp] == 0) {
				board[temp] = tempColor; // update the internal player board
				break;
			}
			temp -= 7; //move one up to check if ok to place token
		}
	}

	/**
	 * UNSET - Unsets last set token
	 * @return void
	 */
	public void unset() {
		drawCounter--;
		int temp = lastMove; // +(7*5); // position at the top of a column
		for (int j = 0;j <= 5; j++) {
			if(board[temp] != 0) {
				board[temp] = 0; // update the internal player board
				break;
			}
			temp += 7; //move one down to check if ok to place token
		}
	}

	/**
	 * GETTOKEN - Gets token from board
	 * @param column to get token from (0-6)
	 * @return token of either color red(1) or yellow(2)
	 */
	public int getToken(int iColumn) {
		int temp = iColumn; // position at the top of a column
		for (int j = 0;j <= 5; j++) {
			if(board[temp] != 0) {
				return board[temp];
			}
			temp += 7; //move one down to check if ok to get token
		}
		return 3; // not used
	}

	/**
	 * ISGAMEDRAWN - See if game is drawn
	 * @return true if 42 tokens is on board else false
	 */
	public Boolean isgameDrawn() { // returns 0 if game is drawn
		if (drawCounter >= 42) {return true;}
		return false;
	}

	/**
	 * WIN - See if 4 tokens connect
	 * @param color of player to check for - 1 for red, 2 for yellow
	 * @return 1 if 4 tokens connect, 0 if not
	 */
	public int win(int tempColor) { // returns 0 if game is drawn
		int playerColor = tempColor;
		int win=0;
		// CHECK ALL VERTICAL
		for (int i = 0; i <= 6; i++) {
			for (int j = i; j <= i+14; j+=7) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+7) == playerColor &&
					this.get(j+14) == playerColor &&
					this.get(j+21) == playerColor) {
					System.out.println("win VERT.");
					return win=1;
				}
			}
		}

		// CHECK ALL HORIZONTAL
		for (int i = 0; i <= 35; i+=7) {
			for (int j = i; j <= i+3; j++) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+1) == playerColor &&
					this.get(j+2) == playerColor &&
					this.get(j+3) == playerColor) {
					System.out.println("win HORI.");
					return win=1;
				}
			}
		}
		// CHECK ALL DIAGONALLY 1
		// FIRST FROM LOWER LEFT TO UPPER RIGHT
		if (this.get(14)==playerColor&&this.get(22)==playerColor &&
			this.get(30)==playerColor&&this.get(38)==playerColor) {System.out.println("win DIAG.1.1");return win=1;}
		if (this.get(7 )==playerColor&&this.get(15)==playerColor &&
			this.get(23)==playerColor&&this.get(31)==playerColor) {System.out.println("win DIAG.1.2");return win=1;}
		if (this.get(15)==playerColor&&this.get(23)==playerColor &&
			this.get(31)==playerColor&&this.get(39)==playerColor) {System.out.println("win DIAG.1.3");return win=1;}
		if (this.get(0 )==playerColor&&this.get( 8)==playerColor &&
			this.get(16)==playerColor&&this.get(24)==playerColor) {System.out.println("win DIAG.1.4");return win=1;}
		if (this.get( 8)==playerColor&&this.get(16)==playerColor &&
			this.get(24)==playerColor&&this.get(32)==playerColor) {System.out.println("win DIAG.1.5");return win=1;}
		if (this.get(16)==playerColor&&this.get(24)==playerColor &&
			this.get(32)==playerColor&&this.get(40)==playerColor) {System.out.println("win DIAG.1.6");return win=1;}
		if (this.get( 1)==playerColor&&this.get( 9)==playerColor &&
			this.get(17)==playerColor&&this.get(25)==playerColor) {System.out.println("win DIAG.1.7");return win=1;}
		if (this.get( 9)==playerColor&&this.get(17)==playerColor &&
			this.get(25)==playerColor&&this.get(33)==playerColor) {System.out.println("win DIAG.1.8");return win=1;}
		if (this.get(17)==playerColor&&this.get(25)==playerColor &&
			this.get(33)==playerColor&&this.get(41)==playerColor) {System.out.println("win DIAG.1.9");return win=1;}
		if (this.get( 2)==playerColor&&this.get(10)==playerColor &&
			this.get(18)==playerColor&&this.get(26)==playerColor) {System.out.println("win DIAG.1.10");return win=1;}
		if (this.get(10)==playerColor&&this.get(18)==playerColor &&
			this.get(26)==playerColor&&this.get(34)==playerColor) {System.out.println("win DIAG.1.11");return win=1;}
		if (this.get( 3)==playerColor&&this.get(11)==playerColor &&
			this.get(19)==playerColor&&this.get(27)==playerColor) {System.out.println("win DIAG.1.12");return win=1;}

		// THEN FROM UPPER LEFT TO LOWER RIGHT
		if (this.get( 3)==playerColor&&this.get( 9)==playerColor &&
			this.get(15)==playerColor&&this.get(21)==playerColor) {System.out.println("win DIAG.2.1");return win=1;}
		if (this.get( 4)==playerColor&&this.get(10)==playerColor &&
			this.get(16)==playerColor&&this.get(22)==playerColor) {System.out.println("win DIAG.2.2");return win=1;}
		if (this.get(10)==playerColor&&this.get(16)==playerColor &&
			this.get(22)==playerColor&&this.get(28)==playerColor) {System.out.println("win DIAG.2.3");return win=1;}
		if (this.get( 5)==playerColor&&this.get(11)==playerColor &&
			this.get(17)==playerColor&&this.get(23)==playerColor) {System.out.println("win DIAG.2.4");return win=1;}
		if (this.get(11)==playerColor&&this.get(17)==playerColor &&
			this.get(23)==playerColor&&this.get(29)==playerColor) {System.out.println("win DIAG.2.5");return win=1;}
		if (this.get(17)==playerColor&&this.get(23)==playerColor &&
			this.get(29)==playerColor&&this.get(35)==playerColor) {System.out.println("win DIAG.2.6");return win=1;}
		if (this.get( 6)==playerColor&&this.get(12)==playerColor &&
			this.get(18)==playerColor&&this.get(24)==playerColor) {System.out.println("win DIAG.2.7");return win=1;}
		if (this.get(12)==playerColor&&this.get(18)==playerColor &&
			this.get(24)==playerColor&&this.get(30)==playerColor) {System.out.println("win DIAG.2.8");return win=1;}
		if (this.get(18)==playerColor&&this.get(24)==playerColor &&
			this.get(30)==playerColor&&this.get(36)==playerColor) {System.out.println("win DIAG.2.9");return win=1;}
		if (this.get(13)==playerColor&&this.get(19)==playerColor &&
			this.get(25)==playerColor&&this.get(31)==playerColor) {System.out.println("win DIAG.2.10");return win=1;}
		if (this.get(19)==playerColor&&this.get(25)==playerColor &&
			this.get(31)==playerColor&&this.get(37)==playerColor) {System.out.println("win DIAG.2.11");return win=1;}
		if (this.get(20)==playerColor&&this.get(26)==playerColor &&
			this.get(32)==playerColor&&this.get(38)==playerColor) {System.out.println("win DIAG.2.12");return win=1;}
		return win;
	}

	/**
	 * WIN3 - See if 3 tokens connect
	 * @param color of player to check for - 1 for red, 2 for yellow
	 * @return 1 if 3 tokens connect, 0 if not
	 */
	public int win3(int tempColor) { // returns 0 if game is drawn
		int playerColor = tempColor;
		System.out.println("win3 playerColor: " +playerColor);
		int win=0;
		// CHECK ALL VERTICAL
		for (int i = 0; i <= 6; i++) {
			for (int j = i; j <= i+14; j+=7) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+7) == playerColor &&
					this.get(j+14) == playerColor) {
					win=1;
				}
			}
		}


		// CHECK ALL HORIZONTAL
		for (int i = 0; i <= 35; i+=7) {
			for (int j = i; j <= i+3; j++) { // 3 possible 4connect's
				if (this.get(j) == playerColor &&
					this.get(j+1) == playerColor &&
					this.get(j+2) == playerColor) {
					win=1;
				}
			}
		}
		// CHECK ALL DIAGONALLY 1
		// FIRST FROM LOWER LEFT TO UPPER RIGHT
		if (this.get(14)==playerColor&&this.get(22)==playerColor &&
			this.get(30)==playerColor) {win=1;}
		if (this.get(7 )==playerColor&&this.get(15)==playerColor &&
			this.get(23)==playerColor) {win=1;}
		if (this.get(15)==playerColor&&this.get(23)==playerColor &&
			this.get(31)==playerColor) {win=1;}
		if (this.get(0 )==playerColor&&this.get( 8)==playerColor &&
			this.get(16)==playerColor) {win=1;}
		if (this.get( 8)==playerColor&&this.get(16)==playerColor &&
			this.get(24)==playerColor) {win=1;}
		if (this.get(16)==playerColor&&this.get(24)==playerColor &&
			this.get(32)==playerColor) {win=1;}
		if (this.get( 1)==playerColor&&this.get( 9)==playerColor &&
			this.get(17)==playerColor) {win=1;}
		if (this.get( 9)==playerColor&&this.get(17)==playerColor &&
			this.get(25)==playerColor) {win=1;}
		if (this.get(17)==playerColor&&this.get(25)==playerColor &&
			this.get(33)==playerColor) {win=1;}
		if (this.get( 2)==playerColor&&this.get(10)==playerColor &&
			this.get(18)==playerColor) {win=1;}
		if (this.get(10)==playerColor&&this.get(18)==playerColor &&
			this.get(26)==playerColor) {win=1;}
		if (this.get( 3)==playerColor&&this.get(11)==playerColor &&
			this.get(19)==playerColor) {win=1;}

		// THEN FROM UPPER LEFT TO LOWER RIGHT
		if (this.get( 3)==playerColor&&this.get( 9)==playerColor &&
			this.get(15)==playerColor) {win=1;}
		if (this.get( 4)==playerColor&&this.get(10)==playerColor &&
			this.get(16)==playerColor) {win=1;}
		if (this.get(10)==playerColor&&this.get(16)==playerColor &&
			this.get(22)==playerColor) {win=1;}
		if (this.get( 5)==playerColor&&this.get(11)==playerColor &&
			this.get(17)==playerColor) {win=1;}
		if (this.get(11)==playerColor&&this.get(17)==playerColor &&
			this.get(23)==playerColor) {win=1;}
		if (this.get(17)==playerColor&&this.get(23)==playerColor &&
			this.get(29)==playerColor) {win=1;}
		if (this.get( 6)==playerColor&&this.get(12)==playerColor &&
			this.get(18)==playerColor) {win=1;}
		if (this.get(12)==playerColor&&this.get(18)==playerColor &&
			this.get(24)==playerColor) {win=1;}
		if (this.get(18)==playerColor&&this.get(24)==playerColor &&
			this.get(30)==playerColor) {win=1;}
		if (this.get(13)==playerColor&&this.get(19)==playerColor &&
			this.get(25)==playerColor) {win=1;}
		if (this.get(19)==playerColor&&this.get(25)==playerColor &&
			this.get(31)==playerColor) {win=1;}
		if (this.get(20)==playerColor&&this.get(26)==playerColor &&
			this.get(32)==playerColor) {win=1;}
		return win;
	}

	/**
	 * GETLASTMOVE - Get last move made.
	 * @return last move
	 */
	public int getLastMove(){
		return lastMove;
	}

	// INSTANCE FIELD
	int[] board;
	int[] lastBoard;
	int drawCounter;
	int lastMove;
}


