package JanusBrunkOlsen;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class Connect4Viewer implements ActionListener {
	private static final long serialVersionUID = 1L;

	/**
	 * MAIN - Starts program and does arbitrary getClass()
	 * to stop eclipse from complaining (i.e. why to use
	 * normal text editor instead of giant IDE)
	 * @return void
	 */
	public static void main(String[] args) {
		Connect4Viewer myGame = new Connect4Viewer();
		myGame.getClass();
	}

	/**
	 * CONNECT4VIEWER - Draws the game frame and also acts
	 * as a referee during the actual play
	 * @return void
	 */
	public Connect4Viewer() {
		// FRAME
		frame = new JFrame();

		// PANELS
		JPanel panelMain = new JPanel(); 				// main
		panelMain.setLayout(new BorderLayout());

		JPanel panelMainNorth = new JPanel();			// main.north
		panelMainNorth.setLayout(new GridLayout(6, 7));
		JPanel panelMainSouth = new JPanel();			// main.south
		panelMainSouth.setLayout(new GridLayout(2, 1));

		JPanel panelMainSouthNorth = new JPanel();		// main.south.north
		panelMainSouthNorth.setLayout(new GridLayout(1, 1));
		JPanel panelMainSouthSouth = new JPanel();		// main.south.south
		panelMainSouthSouth.setLayout(new GridLayout(1, 2));

		panelMainSouth.add(panelMainSouthNorth, BorderLayout.NORTH);
		panelMainSouth.add(panelMainSouthSouth, BorderLayout.SOUTH);
		panelMain.add(panelMainNorth, BorderLayout.NORTH);
		panelMain.add(panelMainSouth, BorderLayout.SOUTH);

		// BUTTONS FOR MAIN.NORTH PANEL
		buttonGrid = new JButton[42];
		for (int j = 0; j <= 6; j++) {
			buttonGrid[j] = new JButton(" ");
			panelMainNorth.add(buttonGrid[j]);
			buttonGrid[j].addActionListener(this);
			buttonGrid[j].setEnabled(false);
		}
		for (int j = 7; j <= 41; j++) {
			buttonGrid[j] = new JButton(" ");
			panelMainNorth.add(buttonGrid[j]);
			buttonGrid[j].setEnabled(false);
		}

		// LABEL FOR MAIN.SOUTH.NORTH PANEL
		labelShowGameInstructions = new JLabel("Waiting for user to start the game");
		panelMainSouthNorth.add(labelShowGameInstructions);

		// BUTTONS FOR MAIN.SOUTH.SOUTH PANEL
		buttonStartGame = new JButton("Start The Game");
		buttonStartGame.addActionListener(this);
		panelMainSouthSouth.add(buttonStartGame);
		buttonResetGame = new JButton("Reset The Game");
		buttonResetGame.addActionListener(this);
		panelMainSouthSouth.add(buttonResetGame);

		// ARRANGE AND SHOW FRAME
		frame.add(panelMain);
		frame.setTitle("Connect4");
		frame.setSize(400, 233); // frame width, height
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLocationRelativeTo(null); // make appear in center of screen
		frame.setResizable(false);
		frame.setVisible(true);

		// ASK USER WHAT TYPE OF GAME TO PLAY
		whichTypeGame = IntroQuestion(); // 0=hVsh,1=mVsh,2=mVsm
		if (whichTypeGame == 0) {
			xPlayerHumanRed = new PlayerHuman("Khrushchev");
			xPlayerHumanRed.init(true); // set xPlayerRed to be color red (...)
			xPlayerHumanYellow = new PlayerHuman("Gorbachev");
			xPlayerHumanYellow.init(false); // set xPlayerRed to be color yellow (...)
		}
		if (whichTypeGame == 1) {
			xPlayerMachineRed = new PlayerMachine("AI Khrushchev");
			xPlayerMachineRed.init(true); // set xPlayerRed to be color red (...)
			xPlayerHumanYellow = new PlayerHuman("Human");
			xPlayerHumanYellow.init(false); // set xPlayerRed to be color yellow (...)
		}
		if (whichTypeGame == 2) {
			// not implemented
		}
	}

	/**
	 * INTROQUESTION - Ask user which type of game to start
	 * @return 1 for human vs human, 2 for machine vs human
	 */
	private int IntroQuestion() {
		Object[] options = {"Human vs Human",
                "Human vs Machine",
                "Machine vs Machine (not done)"};
		int i = JOptionPane.showOptionDialog(frame,
				"Please choose which type of game to play!",
				"Connect4 Game Type",
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE,
				null, options, options[2]);
		return i;
	}

	/**
	 * ACTIONPERFORMED - Actionlistener, reacts on input from gui
	 * playing field events are sent on to UpdateGUIAndInformPlayers
	 * @return void
	 */
	public void actionPerformed(ActionEvent e) {
		// START GAME CLICKED
		if (e.getSource() == buttonStartGame) {
			for (int j = 0; j <= 41; j++) {
				buttonGrid[j].setEnabled(true);
			}
			if (whichTypeGame == 0) {
				labelShowGameInstructions.setText("Game Started, Please make your move "
						+ xPlayerHumanRed.name());
			}
			if (whichTypeGame == 1) {
				AIvsHumanGameStart = 1;
				labelShowGameInstructions.setText("Game Started by "
						+ xPlayerMachineRed.name() + ", please make your move " + xPlayerHumanYellow.name());
				int moveTOColumn = xPlayerMachineRed.move();
				UpdateGUIAndInformPlayers(moveTOColumn);
			}

			return;
		}
		// CANCEL GAME CLICKED
		if (e.getSource() == buttonResetGame) {
			ResetGame();
		}
		// A TOP BUTTON CLICKED
		if(e.getSource() == buttonGrid[0]){UpdateGUIAndInformPlayers(0); return;}
		if(e.getSource() == buttonGrid[1]){UpdateGUIAndInformPlayers(1); return;}
		if(e.getSource() == buttonGrid[2]){UpdateGUIAndInformPlayers(2); return;}
		if(e.getSource() == buttonGrid[3]){UpdateGUIAndInformPlayers(3); return;}
		if(e.getSource() == buttonGrid[4]){UpdateGUIAndInformPlayers(4); return;}
		if(e.getSource() == buttonGrid[5]){UpdateGUIAndInformPlayers(5); return;}
		if(e.getSource() == buttonGrid[6]){UpdateGUIAndInformPlayers(6); return;}
	}

	/**
	 * RESETGAME - Resets the game based on either a draw or user input.
	 * @return void
	 */
	private void ResetGame() {
		// RESET THE GUI BOARD
		for (int j = 0; j <= 41; j++) {
			buttonGrid[j].setEnabled(false);
			buttonGrid[j].setText(" ");
		}
		// SET RED AS STARTER
		colorRed = true;
		colorYellow = false;
		color = true;
		actualColor = Color.red;
		// RESET THE BOARDS OF THE TWO INDIVIDUAL PLAYERS
		if (whichTypeGame == 0) {
			xPlayerHumanRed.reset();
			xPlayerHumanYellow.reset();
			labelShowGameInstructions.setText("Waiting for user to start the game");
		}
		if (whichTypeGame == 1) {
			xPlayerMachineRed = null;
			xPlayerHumanYellow = null;

			xPlayerMachineRed = new PlayerMachine("AI Khrushchev");
			xPlayerMachineRed.init(true); // set xPlayerRed to be color red (...)
			xPlayerHumanYellow = new PlayerHuman("Human");
			xPlayerHumanYellow.init(false); // set xPlayerRed to be color yellow (...)

			labelShowGameInstructions.setText("Waiting for user to start the game");
		}
		return;
	}

	/**
	 * UPDATEGUIANDINFORMPLAYERS - takes input from actionPerformed and
	 * updates accordingly, is essentially the mainloop for the game
	 * @return void
	 */
	private void UpdateGUIAndInformPlayers(int i) {
		int temp = i +(7*5); // position at the bottom of a column
		if(!buttonGrid[i].getText().equals(" ")) {return;} // if column is full
		for (int j = 5;j >= 0; j--) {
			if(buttonGrid[temp].getText().equals(" ")) {

				//######################
				//#### HUMAN VS HUMAN ##
				//######################
				if (whichTypeGame == 0) {
					if(color.equals(colorRed)) {
						xPlayerHumanRed.selfInform(i);
						xPlayerHumanYellow.inform(i); // inform() the opponent
						xPlayerHumanRed.isDrawnCheck();
					}
					else {
						xPlayerHumanYellow.selfInform(i);
						xPlayerHumanRed.inform(i); // inform() the opponent
						xPlayerHumanYellow.isDrawnCheck();
					}
				}

				//########################
				//#### MACHINE VS HUMAN ##
				//########################
				if (whichTypeGame == 1) { // if machine vs human
					if(color.equals(colorRed)) {
						xPlayerHumanYellow.inform(i); // inform() the opponent
						xPlayerMachineRed.isDrawnCheck();
					}
					else {
						xPlayerHumanYellow.selfInform(i);
						xPlayerMachineRed.inform(i); // inform() the opponent
						xPlayerHumanYellow.isDrawnCheck();
					}
				}
				buttonGrid[temp].setText("O"); // update the GUI
				buttonGrid[temp].setForeground(actualColor);
				break;
			}
			temp -= 7; //move one up to check if ok to place token
		}

		//#############################################
		//#### POST MOVE OPERATIONS - HUMAN VS HUMAN ##
		//#############################################
		if (whichTypeGame == 0) { // if human vs human
			if(color.equals(colorRed)) {
				if (xPlayerHumanRed.win() == 1) {
					ResetGame();
					JOptionPane.showMessageDialog(null, "Human player: "
					+ xPlayerHumanRed.name() + " won this round!",
					"Game finished", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
				color = colorYellow;
				actualColor = Color.yellow;
				labelShowGameInstructions.setText("Please make your move "
						+ xPlayerHumanYellow.name());
			}
			else {
				if (xPlayerHumanYellow.win() == 1) {
					ResetGame();
					JOptionPane.showMessageDialog(null, "Human player: "
					+ xPlayerHumanYellow.name() + " won this round!",
					"Game finished", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
				color = colorRed;
				actualColor = Color.red;
				labelShowGameInstructions.setText("Please make your move "
						+ xPlayerHumanRed.name());
			}
		}

		//###############################################
		//#### POST MOVE OPERATIONS - MACHINE VS HUMAN ##
		//###############################################
		if (whichTypeGame == 1) {
			if(color.equals(colorRed)) {
				if (xPlayerMachineRed.win() == 1) {
					JOptionPane.showMessageDialog(null, "Machine player: "
					+ xPlayerMachineRed.name() + " won this round!",
					"Game finished", JOptionPane.INFORMATION_MESSAGE);
					ResetGame();
					return;
				}
				color = colorYellow;
				actualColor = Color.yellow;
				if (AIvsHumanGameStart != 1) {
				labelShowGameInstructions.setText("Please make your move "
					+ xPlayerHumanYellow.name());
				}
				else {AIvsHumanGameStart = 0;}
			}
			else {
				if (xPlayerHumanYellow.win() == 1) {
					JOptionPane.showMessageDialog(null, "Human player: "
					+ xPlayerHumanYellow.name() + " won this round!",
					"Game finished", JOptionPane.INFORMATION_MESSAGE);
					ResetGame();
					return;
				}
				color = colorRed;
				actualColor = Color.red;
				labelShowGameInstructions.setText("Please make your move "
					+ xPlayerMachineRed.name());
				int moveTOColumn = xPlayerMachineRed.move();
				UpdateGUIAndInformPlayers(moveTOColumn); // ONLY WORKS IF MOVE IS VALID
			}
		}
	}

	// INSTANCE FIELD
	JFrame frame;
	JLabel labelShowGameInstructions;
	JButton buttonStartGame, buttonResetGame, buttonGrid[];
	Boolean colorRed = true;  //  @jve:decl-index=0:
	Boolean colorYellow = false;
	Boolean color = true;  //  @jve:decl-index=0:
	Color actualColor = Color.red;  //  @jve:decl-index=0:
	PlayerHuman xPlayerHumanRed;
	PlayerHuman xPlayerHumanYellow;  //  @jve:decl-index=0:
	PlayerMachine xPlayerMachineRed;  //  @jve:decl-index=0:
	int whichTypeGame;
	int AIvsHumanGameStart;
}
