package JanusBrunkOlsen;

import javax.swing.JOptionPane;

public class PlayerMachine implements Player {

	/**
	 * PlayerMachine - sets the name of a machine player and inits
	 * a new board, could in time be used to implement an interface
	 * where the game had highscore..etc.
	 * @return void
	 */
	public PlayerMachine(String tempName) { // player is initialized with a name
		name = tempName;
	}

	/**
	 * INFORM - is used to let the Connect4Viewer feed the data
	 * from a move back to the -opponent- of the player that made
	 * the current move
	 * @return void
	 */
	public void inform(int i) { // player is informed about the move of the opponent (by GUI or referee)
		maxHeight[i]++;
		if (playerColor == 1) {
			xBoard.set(i, 2);
		}
		else {
			xBoard.set(i, 1);
		}
	}

	/**
	 * PRINTCURRENTBOARD - prints current board to the console for debugging
	 * @return void
	 */
	public void printCurrentBoard(String temp){
		System.out.println("playerMachineRED_" + playerColor + "has so board:");
		System.out.println(temp);
		System.out.println("|   "+ xBoard.get(0)+"|   "+xBoard.get(1)+"   "+xBoard.get(2)+"|   "+xBoard.get(4)+"|   "+xBoard.get(4)+"|   "+xBoard.get(5)+"|   "+xBoard.get(6)+"|");
		System.out.println("|   "+ xBoard.get(7)+"|   "+xBoard.get(8)+"   "+xBoard.get(9)+"|   "+xBoard.get(10)+"|   "+xBoard.get(11)+"|   "+xBoard.get(12)+"|   "+xBoard.get(13)+"|");
		System.out.println("|   "+ xBoard.get(14)+"|   "+xBoard.get(15)+"   "+xBoard.get(16)+"|   "+xBoard.get(17)+"|   "+xBoard.get(18)+"|   "+xBoard.get(19)+"|   "+xBoard.get(20)+"|");
		System.out.println("|   "+ xBoard.get(21)+"|   "+xBoard.get(22)+"   "+xBoard.get(23)+"|   "+xBoard.get(24)+"|   "+xBoard.get(25)+"|   "+xBoard.get(26)+"|   "+xBoard.get(27)+"|");
		System.out.println("|   "+ xBoard.get(28)+"|   "+xBoard.get(29)+"   "+xBoard.get(30)+"|   "+xBoard.get(31)+"|   "+xBoard.get(32)+"|   "+xBoard.get(33)+"|   "+xBoard.get(34)+"|");
		System.out.println("|   "+ xBoard.get(35)+"|   "+xBoard.get(36)+"   "+xBoard.get(37)+"|   "+xBoard.get(38)+"|   "+xBoard.get(39)+"|   "+xBoard.get(40)+"|   "+xBoard.get(41)+"|");
	}

	/**
	 * INIT - initializes the player, true for red false for yellow,
	 * in this implementation this translates to 1 for red and 2
	 * for yellow
	 * @return void
	 */
	public void init(Boolean color) { // red is TRUE, yellow is FALSE
		xBoard = new Board();
		if (color.equals(true)) {
		playerColor = 1;
		opponentColor = 2;
		}
		else {opponentColor = 1; playerColor = 2;}
		maxHeight = new int[]{0,0,0,0,0,0,0};
	}

	/**
	 * MOVE - tells the player to move, initializes AI, updates own board
	 * and finally returns move to Connect4Viewer/referee
	 * @return 0 always
	 */
	public int move() { // player is asked to make a move (by GUI or referee)
		scoreAI = new int[]{0,0,0,0,0,0,0}; // scoreboard
		for (int i = 0; i < 7;i++){scoreAI[i] = 0;}

		for (int i = 0; i <= 6;i++) {
			if (maxHeight[i] < 6 && WinInNext(i)) {
				System.out.println("WinInNext on " + i);
				scoreAI[i] += 50;}
			if (maxHeight[i] < 6 && BlockInNext(i)) {
				System.out.println("BlockInNext on " + i);
				scoreAI[i] += 20;}
			if (maxHeight[i] < 6 && Block3InNext(i)) {
				System.out.println("BlockIn3Next on " + i);
				scoreAI[i] += 12;}
			if (maxHeight[i] < 6 && LooseInNextIfSet(i)) {
				System.out.println("LooseInNextIfSet on " + i);
				scoreAI[i] -= 30;}
			if (maxHeight[i] < 6 && Connect3Next(i)) {
				System.out.println("Connect3Next on " + i)
				;scoreAI[i] += 11;}
			if (maxHeight[i] < 6 && Connect2HorizontalRightInNext(i)) {
				System.out.println("Connect2HorizontalRightInNext on " + i);
				scoreAI[i] += 2;}
			if (maxHeight[i] < 6 && Connect2HorizontalLeftInNext(i)) {
				System.out.println("Connect2HorizontalLeftInNext on " + i);
				scoreAI[i] += 2;}
			if (maxHeight[i] < 6 && Connect2VerticalInNext(i)) {
				System.out.println("Connect2VerticalInNext on " + i);
				scoreAI[i] += 4;}
		}
		int bestscoreAI = ReturnBestScoreColumn(scoreAI);
		if (maxHeight[bestscoreAI] >= 5) {
			for (int i = 0; i <= 6;i++) {
				if (maxHeight[i] < 6){xBoard.set(i, playerColor);maxHeight[i]++;return i;}
			}
		}
		System.out.println("BESTSCOREAI at column: " + bestscoreAI);
		xBoard.set(bestscoreAI, playerColor);
		this.printCurrentBoard("testing, after set");
		maxHeight[bestscoreAI]++;
		return bestscoreAI;
	}

	/**
	 * RETURNBESTSCORECOLUMN - returns the column that the AI has
	 * rated highest
	 * @param array to evaluate
	 * @return column
	 */
	public int ReturnBestScoreColumn(int[] scoreAI) {
		int highest = scoreAI[0];
		int index = 0;
		for (int i = 0; i <= scoreAI.length-1; i++){
			System.out.println("scoreAI["+i+"]: " + scoreAI[i]);
			if (scoreAI[i] > highest){highest = scoreAI[i]; index = i;}
		}
		return index;
	}

	/**
	 * CONNECT2VERTICALINNEXT - returns a column that allows the
	 * ai to connect to two tokens vertically
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect2VerticalInNext(int iColumn) {
		int bottom = xBoard.getToken(iColumn);
		xBoard.set(iColumn, playerColor);
		if (bottom == xBoard.getToken(iColumn)) {
			xBoard.unset();
			return true;
		}
		xBoard.unset();
		return false;
	}

	/**
	 * CONNECT2HORIZONTALLEFTINNEXT - returns a column that allows the
	 * ai to connect to two tokens horizontally (to the left of current
	 * token)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect2HorizontalLeftInNext(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (iColumn >= 1 && iColumn <= 6) {
			if (xBoard.getToken(iColumn) == xBoard.getToken(iColumn-1)) {
				xBoard.unset();
				return true;
			}
		}
		xBoard.unset();
		return false;
	}

	/**
	 * CONNECT2HORIZONTALRIGHTINNEXT - returns a column that allows the
	 * ai to connect to two tokens horizontally (to the right of current
	 * token)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect2HorizontalRightInNext(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (iColumn <= 5) {
			if (xBoard.getToken(iColumn) == xBoard.getToken(iColumn+1)) {
				xBoard.unset();
				return true;
			}
		}
		xBoard.unset();
		return false;
	}

	/**
	 * CONNECT3NEXT - returns a column that allows the
	 * ai to connect to two tokens (making it 3 connected tokens..)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Connect3Next(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (xBoard.win3(playerColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * BLOCK3NEXT - returns a column that allows the
	 * ai to block to two tokens (from becoming 3)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean Block3InNext(int iColumn) {
		xBoard.set(iColumn, opponentColor);
		if (xBoard.win3(opponentColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * BLOCKINNEXT - returns a column that allows the
	 * ai to block a opponent win
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean BlockInNext(int iColumn) {
		xBoard.set(iColumn, opponentColor);
		if (xBoard.win(opponentColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * BLOCKINNEXT - returns a column that allows the
	 * ai to win
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean WinInNext(int iColumn) {
		xBoard.set(iColumn, playerColor);
		if (xBoard.win(playerColor) == 1) {xBoard.unset();return true;}
		else {xBoard.unset();return false;}
	}

	/**
	 * LOOSEINNEXTIFSET - returns a column that allows the
	 * ai to know where -not- to put a token (because it
	 * would cause it to loose)
	 * @param array column to evaluate
	 * @return column that allows connection
	 */
	public Boolean LooseInNextIfSet(int iColumn) {
		xBoard.set(iColumn, playerColor);
		xBoard.set(iColumn, opponentColor);
		if (xBoard.win(opponentColor) == 1) {xBoard.unset();xBoard.unset();return true;}
		else {xBoard.unset();xBoard.unset();return false;}
	}

	/**
	 * ISDRAWNCHECK - checks if there is a win on the board associated
	 * to the player, relies on "isgameDrawn" function in board class
	 * @return void
	 */
	public void isDrawnCheck(){
		if (xBoard.isgameDrawn()) {
			JOptionPane.showMessageDialog(null, "Game is over, no winner.",
			"Game drawn", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * WIN - checks if there is a win on the board associated
	 * to the player, relies on "win" function in board class
	 * @return 1 if win, 0 if not
	 */
	public int win() { // player checks if game be won
		int win=0;
		win = xBoard.win(playerColor);
		return win;
	}

	/**
	 * RESET - makes a new board for the player
	 * @return void
	 */
	public void reset() { // return the name of the player
		xBoard = new Board();
	}

	/**
	 * NAME -  returns the name of the player
	 * @return name, string
	 */
	public String name() { // return the name of the player
		return name;
	}

	// INSTANCE FIELD
	String name;
	Board xBoard;
	int playerColor;
	int opponentColor;
	int win = 0;
	Boolean firstround = true;
	int[] scoreAI;
	int[] maxHeight;
}
