package MaiSun;

public class Board {
	private String[][] board;
	private int numberOfTokens;
	private int lastMove;
	private int redTokens = 0;
	private int yellowTokens = 0;

	public Board() {
		reset();
	}

	public void reset() {
		board = new String[7][6];
		numberOfTokens = 0;
		lastMove = -1;
	}

	public void insert(int column, boolean colour) throws IllegalMove {
		if (isValidMove(column) && (redTokens-yellowTokens>=0)) {
			int nextPos = nextFreePos(column);
			board[column][nextPos] = colour + "";
			numberOfTokens++;
			lastMove = column;
			if(colour){
				redTokens++;
			}
			else{
				yellowTokens++;
			}
		} else {
			throw new IllegalMove(column);
		}
	}

	public boolean isValidMove(int column) {
		return column >= 0 && column < 7
				&& board[column][board[column].length - 1] == null;
	}

	public String[][] getBoardLayout() {
		return board;
	}

	public boolean win() {
		int row = 5;
		int nextFree = nextFreePos(lastMove);
		if (nextFree != -1) {
			row = nextFreePos(lastMove) - 1;
		}
		String color = board[lastMove][row];
		if (this.countTokensInColumn(lastMove, row, color) >= 4
				|| this.countTokensInRow(lastMove, row, color) >= 4
				|| this.countTokensInDiagonalX(lastMove, row, color) >= 4
				|| this.countTokensInDiagonalY(lastMove, row, color) >= 4) {
			return true;
		}

		return false;
	}

	public boolean tie() {
		return this.numberOfTokens == 42 && !win();
	}

	public int countTokensInRow(int column, int row, String color) {
		int count = 1;
		int i = column - 1;
		while (i >= 0) {
			if (board[i][row] != null && board[i][row].equals(color)) {
				count++;
				i--;
			} else {
				break;
			}
		}
		i = column + 1;
		while (i < 7) {
			if (board[i][row] != null && board[i][row].equals(color)) {
				count++;
				i++;
			} else {
				break;
			}
		}
		return count;
	}

	public int countTokensInColumn(int column, int row, String color) {
		int count = 1;
		int i = row - 1;
		while (i >= 0) {
			if (board[column][i] != null && board[column][i].equals(color)) {
				count++;
				i--;
			} else {
				break;
			}
		}
		i = row + 1;
		while (i < 6) {
			if (board[column][i] != null && board[column][i].equals(color)) {
				count++;
				i++;
			} else {
				break;
			}
		}
		return count;
	}

	public int countTokensInDiagonalX(int column, int row, String color) {
		int count = 1;
		int i = row - 1;
		int j = column - 1;
		while (i >= 0 && j >= 0) {
			if (board[j][i] != null && board[j][i].equals(color)) {
				count++;
				i--;
				j--;
			} else {
				break;
			}
		}
		i = row + 1;
		j = column + 1;
		while (i < 6 && j < 7) {
			if (board[j][i] != null && board[j][i].equals(color)) {
				count++;
				i++;
				j++;
			} else {
				break;
			}
		}
		return count;
	}

	public int countTokensInDiagonalY(int column, int row, String color) {
		int count = 1;
		int i = row + 1;
		int j = column - 1;
		while (i < 6 && j >= 0) {
			if (board[j][i] != null && board[j][i].equals(color)) {
				count++;
				i++;
				j--;
			} else {
				break;
			}
		}
		i = row - 1;
		j = column + 1;
		while (i >= 0 && j < 7) {
			if (board[j][i] != null && board[j][i].equals(color)) {
				count++;
				i--;
				j++;
			} else {
				break;
			}
		}
		return count;
	}

	private int nextFreePos(int column) {
		for (int i = 0; i < board[column].length; i++) {
			if (board[column][i] == null) {
				return i;
			}
		}
		return -1;
	}

	public void print() {
		for (int i = board[0].length - 1; i >= 0; i--) {
			String row = "";
			for (int j = 0; j < board.length; j++) {
				String value = board[j][i];
				if (value == null) {
					row += "N";
				} else {
					if (value.equals("true")) {
						row += "R";
					} else {
						row += "Y";
					}
				}
			}
			System.out.println(row);
		}
	}

	// Used for recursion
	public Board clone() {
		Board board = new Board();
		String[][] content = new String[7][6];
		for (int i = 0; i < content.length; i++) {
			for (int j = 0; j < content[0].length; j++) {
				content[i][j] = this.board[i][j];
			}
		}
		board.redTokens = this.redTokens;
		board.yellowTokens = this.yellowTokens;
		board.board = content;
		board.lastMove = this.lastMove;
		board.numberOfTokens = this.numberOfTokens;
		return board;
	}

	public boolean isColumnFull(int col) {
		return board[col][board[col].length - 1] != null;
	}
	public int evaluateHeuristicValue(boolean player, int one, int two, int three){
		return this.evaluateHeuristicValueForPlayer(player, one, two, three)-this.evaluateHeuristicValueForPlayer(!player, one, two, three);
	}

	public int evaluateHeuristicValueForPlayer(boolean player, int one, int two, int three) {
		// Basically check all possible winning combinations
		// For rows:
		int res = 0;
		for (int i = 0; i < board[0].length; i++) {
			for (int j = 0; j < 4; j++) {
				res += this.getWeightForPlayer(player, this.buildStringSeq(
						board[j][i], board[j+1][i], board[j+2][i],
						board[j+3][i]), one, two, three);
			}
		}
		//For columns:
		for(int i=0; i<board.length;i++){
			for(int j = 0; j<3 ;j++){
				res += this.getWeightForPlayer(player, this.buildStringSeq(
						board[i][j], board[i][j+1], board[i][j+2],
						board[i][j+3]), one, two, three);
			}
		}
		//For diagonal:
		for(int i = 0; i<4; i++){
			for(int j=0;j<3; j++){
				res += this.getWeightForPlayer(player, this.buildStringSeq(
						board[i][j], board[i+1][j+1], board[i+2][j+2],
						board[i+3][j+3]), one, two, three);
			}
		}
		
		for(int i = 6; i>2; i--){
			for(int j=0;j<3; j++){
				res += this.getWeightForPlayer(player, this.buildStringSeq(
						board[i][j], board[i-1][j+1], board[i-2][j+2],
						board[i-3][j+3]), one, two, three);
			}
		}
		//this.print();
		//System.err.println("The Heuristic Value for player "+player+" in current board is "+res);
		return res;
	}

	private String[] buildStringSeq(String v1, String v2, String v3, String v4) {
		String[] res = new String[4];
		res[0] = v1;
		res[1] = v2;
		res[2] = v3;
		res[3] = v4;
		return res;
	}

	private int getWeightForPlayer(boolean player, String[] seq, int one, int two, int three) {
		int playerCount = 0;
		for (int i = 0; i < seq.length; i++) {
			if (seq[i] != null && seq[i].equals(player + "")) {
				playerCount++;
			}
			if (seq[i] != null && seq[i].equals(!player + "")) {
				// Discard this sequence since it's not possible to make a win
				playerCount = -1;
				break;
			}
		}
		switch (playerCount) {
		case -1: {
			return 0;
		}
		case 0: {
			return 0;
		}
		case 1: {
			return one;
		}
		case 2: {
			return two;
		}
		case 3: {
			return three;
		}
		}
		return 0;
	}
	
	public int getTokenNumber(){
		return this.numberOfTokens;
	}
	public int getLastMove(){
		return this.lastMove;
	}
}
