package MaiSun;

import Player.Player;

public class MaiSun implements Player {
	private Board board;
	private Boolean color;
	//For depth 6, it takes 6-8 second to complete calculation
	//For depth 7, it takes about 20-25 seconds to complete calculation.
	private int depth = 6;
	//The weights are used to configure the player to be more aggressive or defensive
	private int threeWeight = 14;
	private int twoWeight = 3;
	private int oneWeight = 1;
	public MaiSun(){
	}
	//You can ajust the player to be more aggressive or defensive
	//default value is 1, 3, 14
	//If you want it more defensive, please use 1, 4, 14
	public MaiSun(int one, int two, int three){
		oneWeight = one;
		twoWeight = two;
		threeWeight=three;
	}

	public void inform(int i) {
		try {
			this.board.insert(i, !color);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void init(Boolean color) {
		this.board = new Board();
		this.color = color;
	}

	public int move() {
		int alpha = Integer.MIN_VALUE;
		int pos = -1;
		for (int i = 0; i < board.getBoardLayout().length; i++) {
			Board clone = board.clone();
			if (!clone.isColumnFull(i)) {
				try {
					clone.insert(i, this.color);
					// Wins!!!!!!
					if (clone.win()) {
						return i;
					}
					// Game ties, too bad......
					else if (clone.tie()) {
						return i;
					} else {
						int value = this.calculateValueForMin(clone, alpha, 0);
						// Apply alpha-beta algorithm
						if (value > alpha) {
							alpha = value;
							// remember position
							pos = i;
						}

					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		try {
			this.board.insert(pos, this.color);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pos;
	}

	public String name() {
		return "Mai Sun with weight "+this.oneWeight+":"+this.twoWeight+":"+this.threeWeight;
	}

	// Assume win = 1000
	// lose = -1000
	// draw = 0
	public int calculateValueForMax(Board board, int beta, int currDepth) {
		int alpha = Integer.MIN_VALUE;
		for (int i = 0; i < board.getBoardLayout().length; i++) {
			Board clone = board.clone();
			if (!clone.isColumnFull(i)) {
				try {
					clone.insert(i, this.color);
					if (clone.win()) {
						return 1000;
					}
					// Game ties, still too bad......
					else if (clone.tie()) {
						return 0;
					} else {
						if (currDepth < this.depth) {
							int value = this.calculateValueForMin(clone, alpha,
									currDepth + 1);
							alpha = Math.max(value, alpha);
							if (alpha >= beta) {
								return alpha;
							}
						} else {
							return clone.evaluateHeuristicValue(this.color, this.oneWeight, this.twoWeight, this.threeWeight);
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return alpha;
	}

	public int calculateValueForMin(Board board, int beta, int currDepth) {

		int alpha = Integer.MAX_VALUE;
		for (int i = 0; i < board.getBoardLayout().length; i++) {
			Board clone = board.clone();
			if (!clone.isColumnFull(i)) {
				try {
					clone.insert(i, !this.color);
					if (clone.win()) {
						return -1000;
					}
					// Game ties, too bad......
					else if (clone.tie()) {
						return 0;
					} else {
						if (currDepth < this.depth) {
							int value = this.calculateValueForMax(clone, alpha,
									currDepth + 1);
							alpha = Math.min(value, alpha);
							if (alpha <= beta) {
								return alpha;
							}
						} else {
							return clone.evaluateHeuristicValue(this.color, this.oneWeight, this.twoWeight, this.threeWeight);
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return alpha;
	}

}
