package Marguerite;
public class Board 
{
	private int n;
	private int[][] board;
	private static final int ROWS = 6;
	private static final int COLUMNS = 7;

	/**
	 * Constructs a board 
	 */
	public Board()
	{
		//Initialize the board.
		board = new int[COLUMNS][ROWS];
		n = 0;
		//Fill with the board with "0" 
		//(1 is the value of red, -1 is the value of yellow)
		for (int i = 0; i<ROWS; i++)
		{
			for(int j = 0; j < COLUMNS; j++)
				board[j][i] = 0;
		}
	}
	
	/** 
	 * @param col 	Column to put token into
	 * @param player color of the player
	 * @throws IllegalMove 
	 */
	public void insert(int col, boolean playerP) 
								throws IllegalMove
	{	
		int player;
		if (playerP)
		{player = 1;}
		else{
			player = -1;}
		
		if(this.isValid(col))
		{
			for (int row = ROWS-1; row>= 0; row--)
			{
				if(board[col][row] == 0)
				{
					board[col][row] = player;
					break;
				}
			}
		}
		else
		{
			throw (new IllegalMove(col));
		}
	}
	
	//Is only in program to predict the future moves
	public void insert(int col) throws IllegalMove
	{		
		
		
		int player;
		if ((n++)%2==0)
		{player = 1;}
		else{
			player = -1;}
		
		if(this.isValid(col))
		{
			for (int row = ROWS-1; row>= 0; row--)
			{
				if(board[col][row] == 0)
				{
					board[col][row] = player;
					break;
				}
			}
		}
		else
		{
			throw (new IllegalMove(col));
		}
	}
	
	//Check if move is legal
	Boolean isValid(int col)
	{
		if (col <= 6)
		{
			if(!(board[col][0]== -1)||(board[col][0]==1))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else return false;
	}
	
	public boolean tie()
	{
		boolean tiePlay = true;
		
		for (int row = 0; row<ROWS; row++)
		{
			for(int col = 0; col < COLUMNS; col++)
			{
				if((board[col][row]==0))
					{
					tiePlay = false;
					}
			}
		}
		return tiePlay;
	}

	/**
	 * Check if the board has a winning situation
	 * Goes through all indexes of the board to check whether the index 
	 * could be the first token in a row of 4 in the horizontal, 
	 * the vertical or the diagonal direction.
     * If all four are the color of the same - we have a winner
	 */
	public boolean win(boolean pl)
	{
		int player;
		if (pl)
		{player = 1;}
		else{
			player = -1;}
		
		for(int row = 0; row < ROWS; row++)
		{ 
			for (int col = 0; col < COLUMNS; col++)
			{ 
				//Goes through all rows to check if 
				//there is a winning situation
				if ((row+3) < ROWS)
				{
					if (board[col][row] == player && 
							board[col][row+1] == player &&
							board[col][row+2] == player && 
							board[col][row+3] == player)
					{
						return true;
					}
				}
				
				//Goes through all columns to check 
				//if there is a winning situation
				if((col+3) < COLUMNS)
				{
					if (board[col][row] == player && 
							board[col+1][row] == player &&
							board[col+2][row] == player && 
							board[col+3][row] == player)
					{
						return true;
					}
				}
				
				//Goes through all diagonals to check 
				//if there is a winning situation
				if((row + 3) < ROWS && (col+3) < COLUMNS)
				{
					if(board[col][row] == player && 
							board[col+1][row+1] == player &&
							board[col+2][row+2] == player && 
							board[col+3][row+3] == player)
					{
						return true;
					}
				}
				
				//Goes through all diagonals (opposite) to check 
				//if there is a winning situation
				 if ((row - 3)> 0 && (col+3) < COLUMNS )
				{
					if(board[col][row] == player && 
							board[col+1][row-1] == player &&
							board[col+2][row-2] == player && 
							board[col+3][row-3] == player)
					{
						return true;
					}
				}
			}
		}
		return false;	
	}
	
	//Same as previous - but doesn't need any parameter
	public boolean isWin()
	{
		int player;
		if ((n++)%2==0)
		{
			player = 1;
		}
		else
		{
			player = -1;
		}
		
		for(int row = 0; row < ROWS; row++)
		{ 
			for (int col = 0; col < COLUMNS; col++)
			{ 
				//Goes through all rows to check 
				//if there is a winning situation
				if ((row+3) < ROWS)
				{
					if (board[col][row] == player && 
							board[col][row+1] == player &&
							board[col][row+2] == player && 
							board[col][row+3] == player)
					{
						return true;
					}
				}
				
				//Goes through all columns to check 
				//if there is a winning situation
				if((col+3) < COLUMNS)
				{
					if (board[col][row] == player && 
							board[col+1][row] == player &&
							board[col+2][row] == player && 
							board[col+3][row] == player)
					{
						return true;
					}
				}
				
				//Goes through all diagonals to check 
				//if there is a winning situation
				if((row + 3) < ROWS && (col+3) < COLUMNS)
				{
					if(board[col][row] == player && 
							board[col+1][row+1] == player &&
							board[col+2][row+2] == player && 
							board[col+3][row+3] == player)
					{
						return true;
					}
				}
				
				//Goes through all diagonals (opposite) to check 
				//if there is a winning situation
				 if ((row - 3)> 0 && (col+3) < COLUMNS )
				{
					if(board[col][row] == player && 
							board[col+1][row-1] == player &&
							board[col+2][row-2] == player && 
							board[col+3][row-3] == player)
					{
						return true;
					}
				}
			}
		}
		return false;	
	}
	
	//Gets the current board situation 
	//and return it in a double array
	public int[][] getBoard()
	{	
		return board;
	}
	
	
	public Board resetGame()
	{
		Board newBoard = new Board();
		return newBoard;
	}
	
	
	//Prints the game on the console
	public void print()
	{		
			System.out.print("| " + board[0][0]);
			System.out.print(" | " + board[1][0]);
			System.out.print(" | " + board[2][0]);
			System.out.print(" | " + board[3][0]);
			System.out.print(" | " + board[4][0]);
			System.out.print(" | " + board[5][0]);
			System.out.print(" | " + board[6][0] + " |\n");
			System.out.print("| " + board[0][1]);
			System.out.print(" | " + board[1][1]);
			System.out.print(" | " + board[2][1]);
			System.out.print(" | " + board[3][1]);
			System.out.print(" | " + board[4][1]);
			System.out.print(" | " + board[5][1]);
			System.out.print(" | " + board[6][1] + " |\n");
			System.out.print("| " + board[0][2]);
			System.out.print(" | " + board[1][2]);
			System.out.print(" | " + board[2][2]);
			System.out.print(" | " + board[3][2]);
			System.out.print(" | " + board[4][2]);
			System.out.print(" | " + board[5][2]);
			System.out.print(" | " + board[6][2] + " |\n");
			System.out.print("| " + board[0][3]);
			System.out.print(" | " + board[1][3]);
			System.out.print(" | " + board[2][3]);
			System.out.print(" | " + board[3][3]);
			System.out.print(" | " + board[4][3]);
			System.out.print(" | " + board[5][3]);
			System.out.print(" | " + board[6][3] + " |\n");
			System.out.print("| " + board[0][4]);
			System.out.print(" | " + board[1][4]);
			System.out.print(" | " + board[2][4]);
			System.out.print(" | " + board[3][4]);
			System.out.print(" | " + board[4][4]);
			System.out.print(" | " + board[5][4]);
			System.out.print(" | " + board[6][4] + " |\n");
			System.out.print("| " + board[0][5]);
			System.out.print(" | " + board[1][5]);
			System.out.print(" | " + board[2][5]);
			System.out.print(" | " + board[3][5]);
			System.out.print(" | " + board[4][5]);
			System.out.print(" | " + board[5][5]);
			System.out.print(" | " + board[6][5] + " |\n \n");
		}
	
}
