package NaderSalas;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Ellipse2D;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.Timer;

class CellComponent extends JComponent{ 
	/** This is GUI board space component 6x7 */
	private Ellipse2D.Double circle; /** The surface of this component in which it's color changes.*/
	private Color color; /** The component's surface color. This visualize "Token"  */
	public CellComponent(Color color) /** Initialize component to be empty (with background(empty)
	 color). */
	{
		this.color=color;
	}
	public void paintComponent(Graphics m) /** Implementation of JComponent paintComponent interface. */
	{
		Graphics2D g2 = (Graphics2D) m;
		this.setBorder(BorderFactory.createEtchedBorder());
		int w=this.getWidth()-20;
		int h=this.getHeight()-20;
		circle = new Ellipse2D.Double(10,10,w,h);
		g2.setColor(color);
		g2.fill(circle);
		g2.draw(circle);		
	}
	public void setColor(Color color) /** Change space color to this color.*/
	{
		this.color=color;
		repaint();
	}
}

class DropTokenComponent extends CellComponent{ 
	/** This is specialized CellComponent only for human paler 1x6 */
	private int player=0,Col=0;
	private Color[] foreColor= new Color[2]; /** Foreground color of space changes to player's color */
	private Color backColor; /** Empty spaces are spaces with backColor */
	private boolean noColor=false; /** If true, then surface color is backColor and 
	mouse activities have no effect on changing color of component's surface.  */
	public DropTokenComponent(Color backColor,Color colorPlayer1,Color colorPlayer2) /** Initialize the 
	component with backColor and players color */
	{
		super(backColor);
		this.backColor=backColor;
		foreColor[0]=colorPlayer1;
		foreColor[1]=colorPlayer2;
	}
	public void setForeColor() /** Sets foreground color for current player */
	{
		if (!noColor){
			super.setColor(foreColor[player]);	
		}
	}
	public void setBackColor() /** Sets background color to active color */
	{
		super.setColor(backColor);
	}
	public void switchColor() /** Toggles players color */
	{
		if (!noColor){
			player=1-player;
		}
	}
	public int getCol() /** Returns the column of UI board for this component */
	{
		return Col;
	}
	public void setCol(int Col) /** Sets the column of this component for UI board */
	{
		this.Col= Col;
	} 
	public void setPlayer(int player) /** Sets the GUI board player. It is used to toggle player color */
	{
		this.player=player;
	}
	public void setNoColor() /** Stops respond to color changing of this component. 
	It is used when a column filled with tokens */
	{
		noColor=true;
		super.setColor(backColor);
		repaint();
	}
	public void setShowColor() /** Starts respond to color changing of this component. */
	{
		noColor=false;
	}
}

class GUI extends JComponent{
	private boolean GameIsOver=false; 
	private int TIMER_INITIAL = 18; /**This is the initial value of seconds to move*/
	private NaderSalas CPlayer;
	private CellComponent[][] spaceComponentArray = new  CellComponent[6][7]; /** This is representation 
	of spaces of board in which token can be placed*/	
	private DropTokenComponent[] dropTokenArray = new  DropTokenComponent[7]; /** This is playing entry 
	for human player */
	private Color[] boardColor= new Color[2]; /** There are two color representing two players */
	private boolean[] boolPlayer = {true,false}; /** Matches 0 to True and 1 to false */
	private Color cellBackColor=new  Color(210,210,210); /** Background color for spaces */
	private Color moveBackColor=new Color(235,235,235);/** Background color for token drop */
	private MouseListener ML;
	public final static Color FIRST_COLOR = Color.RED;
	public final static Color SECOND_COLOR = Color.YELLOW;
	private String[] PlayerColor={"Yellow","Red"}; /** Players name */
	private JLabel turnLabel=new JLabel(); /** Label showing who has the next turn */
	private JLabel showTimer=new JLabel("Time remaining...");
	private JButton newGame = new JButton("Play again");
	private JButton stopGame = new JButton("Stop");
	private JTextField field = new JTextField(3); /** A field holding number of second to drop token */
	private JComboBox selectPlayer = new JComboBox(); /** In this version Human-Human, Human-Computer*/
	private boolean human=true; /** True, Human-Human, false Human-Computer */
	private Timer GUITimer; /** Update GUI remaining time in second */
	private int GUItimeLeft=TIMER_INITIAL; /** Initial time in second to drop token for each player*/
	private GUIBoard board; /** Internal representation of board */
	public GroupMap gMap;
	
	public GUI(Container pane){

		boardColor[0]=FIRST_COLOR;
		boardColor[1]=SECOND_COLOR;
		gMap=new GroupMap();
		board=new GUIBoard();
		ML=createMouseListener();
		this.setLayout(new GridLayout(7,7));
		// make dropToken component ready 
		for (int i=0;i<7;i++){
			dropTokenArray[i]=new DropTokenComponent(moveBackColor,boardColor[0],boardColor[1]);
			dropTokenArray[i].setCol(i);
			add(dropTokenArray[i]);
		}	
		// make space component ready 
		for (int row=0;row<6;row++){
			for (int col=0;col<7;col++){
				spaceComponentArray [row][col]= new CellComponent(cellBackColor);
			}
		}
		// count down- because of GridLayout that automatically starts from top left cell
		for (int row=5;row>=0;row--){ 
				for (int col=0;col<7;col++){
					add(spaceComponentArray [row][col]);
				}
			}
		makePanel(pane);	
		GUITimer = new Timer(1000,new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				GUItimeLeft--;
				showTimer.setText("Time remaining..."+GUItimeLeft);
				if (GUItimeLeft==0){
					GUITimer.stop();
					GUItimeLeft=TIMER_INITIAL;
					gameOver();
					annonceWinner();
				}
			}			
		});
		InitGUI();
	}
	public MouseListener createMouseListener()
	{
		return new MouseListener(){	
			public void mouseExited(MouseEvent e)
			{((DropTokenComponent)e.getComponent()).setBackColor();} 
			public void mouseEntered(MouseEvent e)
			{((DropTokenComponent) e.getComponent()).setForeColor();}
			public void mouseClicked(MouseEvent e)
			{mouseHasBeenClicked((DropTokenComponent)e.getComponent());}
			public void mousePressed(MouseEvent e){}
			public void mouseReleased(MouseEvent e){}
		};
	}
	public void mouseHasBeenClicked (DropTokenComponent dropTokenComponent){
		dropTokenComponent.setBackColor();
		int droptedCol =dropTokenComponent.getCol();
		if(board.humanInform(droptedCol)){ // if it is a legal move
			if(human)
				for(int i=0;i<7;i++) // Toggle GUI board token color
					dropTokenArray[i].switchColor();
			else
				CPlayer.inform(droptedCol);	
			
			int col=board.getBordCol();
			int row=board.getBordRow();
			int p=board.getPlayer();
			//set color of the current space
			spaceComponentArray[row][col].setColor(boardColor[1-p]);
			if (row==5)  // if this is the last token for the current column 
				dropTokenComponent.setNoColor();
			GUItimeLeft=TIMER_INITIAL;
			
			if (check_Winner())
			{
				gameOver();
				return;
			}
			else
				if (board.isTie())
				{
					gameIsTie();
					return;
				}
				else
					dropTokenComponent.setForeColor();
			
		}
		if(!human && board.getPlayer()==0) // Computer playing as first player
		{
			playComputerHuman();
		}
	}			
	public void makePanel(Container pane)
	{
		JPanel panel=new JPanel(new GridLayout(3,1));
		
		JPanel Line1Panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JPanel Line2Panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel Line3Panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JPanel Line4Panel = new JPanel(new FlowLayout(FlowLayout.CENTER));		
		JPanel Line5Panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JPanel Line6Panel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		
		JPanel Line12Panel = new JPanel(new GridLayout(1,2));
		Line12Panel.add(Line1Panel);
		Line12Panel.add(Line2Panel);
		JPanel Line34Panel = new JPanel(new GridLayout(1,2));
		Line34Panel.add(Line3Panel);
		Line34Panel.add(Line4Panel);
		JPanel Line56Panel = new JPanel(new GridLayout(1,2));
		Line56Panel.add(Line5Panel);
		Line56Panel.add(Line6Panel);
		
		newGame.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				int test= (int)Integer.parseInt(field.getText());
				if (test <= 200 && test > 0)
				{
					TIMER_INITIAL=test;
					GUItimeLeft=test;
				}
				else
				{
					GUItimeLeft=TIMER_INITIAL;
					field.setText(String.valueOf(GUItimeLeft));					
				}
				disableItems();
				stopGame.setEnabled(true);
				board.resetBoard();
				InitGUI();
				if(!human && board.getPlayer()==0)
				{
					CPlayer.init(true);
					playComputerHuman();
					for(int i=0;i<7;i++) // Toggle GUI board token color
						dropTokenArray[i].switchColor();
					
				}
			}
		});
		
		Integer.getInteger(field.getText());
	
		stopGame.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(GUITimer.isRunning())
					GUITimer.stop();
				gameOver();
				enableItems();
				stopGame.setEnabled(false);
			}
		}); 
		
		turnLabel.setFont(new Font(turnLabel.getFont().getFontName(),turnLabel.getFont().getStyle(),20));
		turnLabel.setText("Red turn!!");
		
		selectPlayer.addItem("Human with Human");
		selectPlayer.addItem("Human with Computer");
		
		selectPlayer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(((JComboBox)e.getSource()).getSelectedIndex()==0) //if human with human
					human=true;
				else{
					newGame.doClick();
					human=false;
					CPlayer = new NaderSalas();
					CPlayer.init(true);
					CPlayer.initBoard();
					CPlayer.resetBoard();
					board.init(false); // computer is first player
					playComputerHuman();
					for(int i=0;i<7;i++) // Toggle GUI board token color
					dropTokenArray[i].switchColor();
				}
			}
		}); 

		
		Line2Panel.add(showTimer);
		Line4Panel.add(turnLabel);


		Line3Panel.add(new JLabel("Enter number of second for each move: "));
		field.setText("18");
		Line3Panel.add(field);
		
		Line1Panel.add(new JLabel("Select player: "));
		Line1Panel.add(selectPlayer);
		
		Line5Panel.add(newGame);
		Line6Panel.add(stopGame);
		
		panel.add(Line12Panel);
		panel.add(Line34Panel);
		panel.add(Line56Panel);
	
		pane.add(panel,BorderLayout.SOUTH);
	}
	public void gameOver(){
		for (int i=0;i<7;i++)
		{
			dropTokenArray[i].setNoColor();
			dropTokenArray[i].removeMouseListener(ML);
			if(!human) CPlayer.resetBoard();
		}
		GameIsOver=true;				
	}
	public void playComputerHuman()
	{		
		if(board.humanInform(CPlayer.move())){ // if it is a legal move
			int col=board.getBordCol();
			int row=board.getBordRow();
			int p=board.getPlayer();
			if(board.getTurn()==1)
			for(int i=0;i<7;i++) // Toggle GUI board token color
				dropTokenArray[i].switchColor();
			//set color of the current space
			spaceComponentArray[row][col].setColor(boardColor[1-p]);
			if (row==5)  // if this is the last token for the current column 
				dropTokenArray[col].setNoColor();
			GUItimeLeft=TIMER_INITIAL;
			if (check_Winner())
				gameOver();
			else
				if (board.isTie())
					gameIsTie();			
		}
	}
	public void gameIsTie()
	{
		gameOver();
		turnLabel.setText("Game is tie!!");
		GUITimer.stop();
		showTimer.setText(" ");
	}
	public boolean check_Winner(){
		boolean winner=false;
		if (board.isWinner())
			{
				winner = true;
				annonceWinner();
			}
		if(!winner)
			turnLabel.setText(PlayerColor[1-board.getPlayer()]+" turn!!");
		return winner;
	}
	public void annonceWinner()
	{
		turnLabel.setText(PlayerColor[board.getPlayer()]+" won!!");
		GUITimer.stop();
		showTimer.setText(" ");
		enableItems();
		stopGame.setEnabled(false);
	}
	public void enableItems()
	{
		newGame.setEnabled(true);
		field.setEnabled(true);
		selectPlayer.setEnabled(true);
		
	}
	public void disableItems()
	{
		newGame.setEnabled(false);
		field.setEnabled(false);
		selectPlayer.setEnabled(false);
		GameIsOver=false;
	}
	public void InitGUI()	
	{
		/**this method will be also called for new game 
		 * @parameter dropTokenComponent is an array which holds 7 dropTokenComponent
		 */
		for(int i=0;i<7;i++)
		{
			dropTokenArray[i].setShowColor(); //Show color
			dropTokenArray[i].setBackColor();
			dropTokenArray[i].addMouseListener(ML);
			dropTokenArray[i].setPlayer(0);
		}
		for (int y=0;y<6;y++)
			for (int x=0;x<7;x++)
				spaceComponentArray [y][x].setColor(cellBackColor);

		turnLabel.setText("Red turn!!");
		disableItems();
		GUITimer.start();
	}	
}
