package NaderSalas;
import java.util.ArrayList;
import javax.swing.Timer;
import Player.Player;



public class NaderSalas implements Player{
	public final static int TIMER_INITIAL = 18;
	java.util.Random generator;
	
	private int[][] state = new int[2][69]; /** each player has an array of board state */
	private int[][] boardSpace = new int[6][7]; /** 0=player1, 1=player 2, 2=empty */
	private boolean IamFirstPlayer; /** This is to know who is the first player */
	private int turn=0,player=0,row=0,col=0;/** Keeping track of board coordinates, players and turn */
	private Timer boardTimer;  /** To update UI for human players */
	private int timeLeft=TIMER_INITIAL;
	private GameBoard gameBoard; 
	private GroupMap groupMap;
	public GroupMap gMap;

	
	public NaderSalas()
	{
		gMap=new GroupMap();
		initBoard();
		resetBoard();
		generator = new java.util.Random();
	}
	public void init (Boolean color) /** Initialize player- first player->true opponent->false */
	{
		IamFirstPlayer = color;
	}
	public String name () /** Return author*/ 
	{return "Nader Salas";}
	public int move() /** Return computer's move */
	{
		int result=-1;
		if (IamFirstPlayer && turn==0)  // "red" is true meaning that first player
			result= 3;		
		else
			result=getBestMove(player);
		turn++;
		gameBoard.dropToken(result,player); //register the move I have just made
		player=1-player;		
		return result;
	}
	public void inform (int i) /** Informs board the opponent/current-player's move 
	(because return type of "void" this will always be a Computer-opponent's move).
	For human player see "boolean informHuman(int i).*/
	{
		gameBoard.dropToken(i,player); //register the move that Computer opponent has just made
		player=1-player;
	}
	public boolean humanInform (int i) /** Because Carsten's "void inform(int i)" is void!!! */
	//human player make this move
	{
		if (dropToken(i)) // if it is a valid move
		{
			player=1-player;
			return true;
		}
		else
			return false;								
	}
	public void initBoard() /** Initialize the board variables for current game session.*/
	{
		for(int player=0;player<2;player++)
			for(int group=0;group<69;group++)
				state[player][group]=1;
		for(int row=0;row<6;row++)
			for(int col=0;col<7;col++)
				boardSpace[row][col]=2;
		this.gameBoard = new GameBoard(state,boardSpace);
//		groupMap= new GroupMap();
	}
	public void resetBoard() /** Make the board ready for new game.*/
	{		
		//this will also be called for new game
		//initialize state of the board
		turn=player=row=col=0;
		for (int i=0;i<2;i++){
			for(int j=0;j<69;j++){
				gameBoard.state[i][j]=1;
			}
		}
		for (int row=0;row<6;row++){
			for(int col=0;col<7;col++){
				gameBoard.boardSpace[row][col]=2;
			}
		}
	}
	public boolean isWinner() /** Check the internal map for possible winner, if there is one return true.*/
	{
		for(Integer i:groupMap.getGroupMap(this.row,this.col))
		{
			if ((gameBoard.state[this.player][i]*=2)==8)
				return true;
			gameBoard.state[1-this.player][i]=0;
		}
		return false;
	}
	public int getPlayer() /** Returns the current player.*/
	{return this.player;}
	public boolean isTie() /** Return true if the game is tie.*/
	{
		for (int row=0;row<6;row++)
			for(int col=0;col<7;col++)
				if (gameBoard.boardSpace[row][col]==2)
					return false;
		return true;
	}
	public boolean dropToken(int col)/** Update internal arrays and return false if move is not valid. */
	{
		boolean result=false;
		for (int i=0;i<6;i++)
		{
			if(gameBoard.boardSpace[i][col]==2)// empty space
			{
				gameBoard.boardSpace[i][col]=player;
				this.row=i;
				this.col=col;
				player=1-player;				
				result=true;
				break;
			}
		}
		return result;
	}
	
	public int getBestMove(int player) /** Prepare for best move.*/
	{		
		GameBoard gB = new GameBoard(gameBoard.state,gameBoard.boardSpace);
		gB.m.row=gameBoard.m.row;
		gB.m.col=gameBoard.m.col;
		
		SharedResources sr=new SharedResources(18); //18 seconds
		SearchMiniMax sH= new SearchMiniMax(gB,player,sr);
		
		WorkerThread w = new WorkerThread(sr);
		Thread wT = new Thread(w);
		
		Thread sT = new Thread(sH);
	
		sT.start();
		wT.start(); //time starts here
		
		int bMove=-1;
		try{
			bMove=sr.getMove();
		} catch (InterruptedException e){}
		return bMove;
	}

	/** For debugging purpose */
	public int getRandomMove() /** For test.*/
	{
		ArrayList<Integer> randNumbers = new ArrayList<Integer>();
		for (int col=0;col<7;col++)
			for (int row=0;row<6;row++)
				if (gameBoard.boardSpace[row][col] == 2)
				{
					randNumbers.add(new Integer(col));
					break;
				}

		int i=randNumbers.size();
		if (!(i<= 0))
			return randNumbers.get(generator.nextInt(i));
		else
			return -1;
	}
	public int getBordRow() /** Return board row for last play.*/
	{return row;}
	public int getTurn() /** Return turn.*/
	{
		return turn;
	}
	public int getBordCol() /** Return board column for last play.*/
	{return col;}
}