package PeterRaahauge;

import Player.Player;
import java.lang.Math;

public class PeterRaahauge implements Player {
	
		// Constructor
		public PeterRaahauge(){
			board = new Board();
			name = "Speedy Gonzales";
			color = true;
		}
	
		// Interface method 
		public void init(Boolean color){
			this.color = color;
		};
		
		// Interface method
		public void inform(int i){ 
			int tokenCol = i+1;
			board.newToken(tokenCol,!color);
		};

		// Interface method
		public String name() {
			return name;
		}
		
		// Interface method
		public int move() {
			
			// Ratings of each point on the board for the first and second player
			double[][] firstRating = new double[ROWS][COLS];
			double[][] secondRating = new double[ROWS][COLS];
			
			// Temporary ratings for the 16 possible Connect 4 combinations in which
			// each point can engage in. (will be conditional on i,j)
			double[] firstPointRating = new double[16];
			double[] secondPointRating = new double[16];
			
			
			int firstMatch;
			int secondMatch;
			
			// All free points on the board are rated for first and second player 
			for(int i=0; i<ROWS; i++){					// For all rows
				for(int j=0;j<COLS;j++){ 				// For all columns
					if( board.getBoardValue(i,j) > 0){}	//This point is filled already, no rating necessary    
					else{								
						for(int k=0;k<16;k++){			// For all winning combinations
							if(!isInside(i,j,k)){		// If combination not inside board  
								firstPointRating[k] = 0;
								secondPointRating[k] = 0;
							}
							else{	
								// For each of the 16 possibilities, the likelihood of getting
								// 4 in a row is rated.
								firstMatch = nrOfMatch(i,j,k,1);
								secondMatch = nrOfMatch(i,j,k,2);
								if (secondMatch >0) 
									firstPointRating[k]= 0;
								else 
									firstPointRating[k]= Math.pow(firstMatch+1,2*firstMatch); 
								if (firstMatch >0) 
									secondPointRating[k]= 0;
								else 
									secondPointRating[k]= Math.pow(secondMatch+1,2*secondMatch); 
							}
						}
						// The rating of all the 16 combinations are added together
						firstRating[i][j] = 0;
						secondRating[i][j] = 0;
						for(int k=0;k<16;k++){
							firstRating[i][j] += firstPointRating[k];
							secondRating[i][j] += secondPointRating[k];
						}
					}
				}
			}
			
			// A "first move" in each column is rated
			double[] firstColumnValue = new double[7]; 
			double[] secondColumnValue = new double[7]; 
			
			for(int j=0;j<COLS;j++){
				firstColumnValue[j] = 0;
				secondColumnValue[j] = 0;
				
				int firstFree = 0; 
				for(int i=0; i<ROWS; i++){
					if(board.getBoardValue(i,j) == 0){
						firstFree = i;
						break;
					}
				}
					
				for(int i=firstFree; i<ROWS; i+=2){
					firstColumnValue[j] += firstRating[i][j]/Math.sqrt(2*(i-firstFree+1));
					secondColumnValue[j] += secondRating[i][j]/Math.sqrt(2*(i-firstFree+1));
					
					// The future possibilities of the opponent are taken into account 
					if (i+1<ROWS){
						firstColumnValue[j] -= secondRating[i+1][j]/Math.sqrt(2*(i-firstFree+2));
						secondColumnValue[j] -= firstRating[i+1][j]/Math.sqrt(2*(i-firstFree+2));
					}					
				}	
			}
		
			// Make decision: Find best column
			int firstHotColumn = 0;			
			int secondHotColumn = 0;			
			double firstScore = -1000;
			double secondScore = -1000;
			double firstTest;
			double secondTest;
			for(int j=0;j<COLS;j++){
				firstTest = Math.max(firstColumnValue[j],secondColumnValue[j]);
				secondTest = Math.max(secondColumnValue[j],firstColumnValue[j]);
				if(firstTest>firstScore){
					firstHotColumn = j;
					firstScore = firstTest;
				}
				if(secondTest>secondScore){
					secondHotColumn = j;
					secondScore = secondTest;
				}
			}
			
			// Depending on type, report move
			if (color){
				board.newToken(firstHotColumn+1,color);
				return (firstHotColumn);
			}
			else{
				board.newToken(secondHotColumn+1,color);
				return (secondHotColumn);
			}
		}
		
		private int nrOfMatch(int i,int j,int k,int matchValue){
			// Calculates for each i,j point, the number of favorable matches
			// already in the k'th connect 4 combination 
			int count = 0;
			if( board.getBoardValue(i+rIx[k][0],j+cIx[k][0]) == matchValue) count++;
			if( board.getBoardValue(i+rIx[k][1],j+cIx[k][1]) == matchValue) count++;
			if( board.getBoardValue(i+rIx[k][2],j+cIx[k][2]) == matchValue) count++;
			if( board.getBoardValue(i+rIx[k][3],j+cIx[k][3]) == matchValue) count++;
			return count;
		}
		
		private boolean isInside(int i,int j,int k){
			// Checks if connec-4 combination is inside board
			boolean ok = false;
			if(		i+rIx[k][0] >= 0 && i+rIx[k][0] < ROWS &&
					i+rIx[k][1] >= 0 && i+rIx[k][1] < ROWS &&
					i+rIx[k][2] >= 0 && i+rIx[k][2] < ROWS &&
					i+rIx[k][3] >= 0 && i+rIx[k][3] < ROWS 
					&&			
					j+cIx[k][0] >= 0 && j+cIx[k][0] < COLS &&
					j+cIx[k][1] >= 0 && j+cIx[k][1] < COLS &&
					j+cIx[k][2] >= 0 && j+cIx[k][2] < COLS &&
					j+cIx[k][3] >= 0 && j+cIx[k][3] < COLS)
				ok = true;
			return ok;
		}
		
		// Set'ers and get'ers nice to have 
		public Board getBoard(){
			return board;
		}
		public void setName(String name){
			this.name = name;
		}	
		
		private String name;
		private Boolean color;
		private Board board;

		// rIx and cIx are the relative index combinations of the 16 connect-4 possibilities
		private int[][] rIx = 
		{
				{0,1,2,3},
				{-1,0,1,2},
				{-2,-1,0,1},
				{-3,-2,-1,0},
				
				{0,0,0,0},
				{0,0,0,0},
				{0,0,0,0},
				{0,0,0,0},
				
				{0,1,2,3},
				{-1,0,1,2},
				{-2,-1,0,1},
				{-3,-2,-1,0},
				
				{0,1,2,3},
				{-1,0,1,2},
				{-2,-1,0,1},
				{-3,-2,-1,0}
		};
		
		private int[][] cIx = {
				{0,0,0,0},
				{0,0,0,0},
				{0,0,0,0},
				{0,0,0,0},
				
				{0,1,2,3},
				{-1,0,1,2},
				{-2,-1,0,1},
				{-3,-2,-1,0},
				
				{0,-1,-2,-3},
				{1,0,-1,-2},
				{2,1,0,-1},
				{3,2,1,0},
				
				{0,1,2,3},
				{-1,0,1,2},
				{-2,-1,0,1},
				{-3,-2,-1,0}
		};
	
		private static final int ROWS = 6;
		private static final int COLS = 7;	
}
