package SemBlaksmark; 
import Player.Player;
import java.util.ArrayList;

/**
 * An AI player for connect4
 * 
 * @author (Sem Blaksmark) 
 * @version (15-12-2007)
 */
public class SemBlaksmark implements Player
{   
    Boolean color;
    String name;
    
    Boolean[][] currentGame = new Boolean[6][7];    

    public void init(Boolean color)
    {
        this.color = color;
        
        if (color)
        {
            name = "red";
        }
        else
        {
            name = "yellow";
        }
    }
    
    public void inform(int column)
    {
        for (int i = currentGame.length-1; i >= 0; i--)
        {
            if (currentGame[i][column] == null)
            {
                currentGame[i][column] = new Boolean(!color);
                i = 0;
            }
        }
    }    
    
    public String name()
    {
        return name;
    }
    
    public int move()
    {
        State tree = new State(currentGame, color);
        traverse(tree);
        System.out.println(tree.getExpectedMove());
        for (int i = currentGame.length-1; i >= 0; i--)
        {
            if (currentGame[i][tree.getExpectedMove()] == null)
            {
                currentGame[i][tree.getExpectedMove()] = new Boolean(!color);
                i = 0;
            }
        }
        return tree.getExpectedMove();
    }
    
    public int traverse(State tree)
    {
        if (tree.getSubNodes().size() == 0)
        {            
            tree.setValue(eval(tree.getState()));            
            return tree.getValue();
        }
        
        else if (tree.getColor() == color)
        {
            int max = -1000000;
            for (int i = 0; i < tree.getSubNodes().size(); i++)
            {
                if (traverse(tree.getSubNodes().get(i)) > max)
                {                    
                    max = tree.getSubNodes().get(i).getValue();
                    tree.setExpectedMove(tree.getSubNodes().get(i).getMove());
                }
            }            
            tree.setValue(max);            
            return max;
        }
        else
        {           
            int min = 1000000;
            for (int i = 0; i < tree.getSubNodes().size(); i++)
            {
                if (traverse(tree.getSubNodes().get(i)) < min)
                {
                    min = tree.getSubNodes().get(i).getValue();
                    tree.setExpectedMove(tree.getSubNodes().get(i).getMove());
                }
            }
            tree.setValue(min);
            return min;
        }        
    }
    
    public int eval(Boolean[][] state)
    {
        int value = 0;         
        ArrayList<Boolean> temp;
        
        //checking rows                   
        for (int i = 0; i < state.length; i++)
        {
            temp = new ArrayList<Boolean>();
            for (int j = 0; j < state[i].length; j++)
            {
                temp.add(state[i][j]);                
            }
            value += arrayEval(temp);
        }
        
        //Checking columns        
        for (int i = 0; i < state[0].length; i++)
        {
            temp = new ArrayList<Boolean>();
            for (int j = 0; j < state.length; j++)
            {
                temp.add(state[j][i]);
            }
            value += arrayEval(temp);
         }

        //Checking downward diagonals originating on either side of the upper half of the board
        for (int i = 0; i < state.length/2; i++)
        {
            //the left side
            temp = new ArrayList<Boolean>();
            for (int j = 0; j < state.length-i; j++)
            {
                    temp.add(state[i+j][j]);
            }
            value += arrayEval(temp);
            
            //the right side
            temp = new ArrayList<Boolean>();         
            for (int j = 0; j < state.length-i; j++)
            {                
                temp.add(state[i+j][state[i].length-1-j]);
            }
            value += arrayEval(temp);
        }
        
        //Checking diagonals originating on either side of the lower half of the board
        for (int i = state.length-1; i >= state.length/2; i--)
        {            
            //the left side
            temp = new ArrayList<Boolean>();
            for (int j = 0; j <= i; j++)
            {                
                temp.add(state[i-j][j]);
            }
            value += arrayEval(temp);
            
            //The right side
            temp = new ArrayList<Boolean>();
            for (int j = 0; j <= i; j++)
            {                
                temp.add(state[i-j][state[i].length-1-j]);
            }
            value += arrayEval(temp);
        }
        return value; 
    }
    
    private int arrayEval(ArrayList<Boolean> row)
    {   
        int last = -1;
        int count = 0;
        int value = 0;
        int value2 = 0;
        
        //generate a "goodness" value going from left to right through the array
        for (int i = 0; i < row.size(); i++)
        {
            if (row.get(i) == null)
            {                
                count++;
                value++;
            }
            else if (last == -1)
            {
                last = i;
                count++;
                value += 3;
            }
            else if (row.get(i).equals(row.get(last)))
            {
                count++;
                value += 3;
            }
            else
            {
                if (count < 4)
                {
                    last = i;
                    count = 1;
                    value = 3;
                }
                else
                {
                    i = row.size();
                }
            }
        }
        if (last == -1 || count < 4)
        {
            value = 0;
        }        
        else if (!row.get(last))
        {
            value = -value;
        }
        
        //generate a "goodness" value going from right to left through the array
        last = -1;
        count = 0;
        
        for (int i = row.size()-1; i >=0; i--)
        {
            if (row.get(i) == null)
            {                
                count++;
                value2++;
            }
            else if (last == -1)
            {
                last = i;
                count++;
                value2 += 3;
            }
            else if (row.get(i).equals(row.get(last)))
            {
                count++;
                value2 += 3;
            }
            else
            {
                if (count < 4)
                {
                    last = i;
                    count = 1;
                    value2 = 3;
                }
                else
                {
                    i = -1;
                }
            }
        }
        if (last == -1 || count < 4)
        {
            value2 = 0;
        }        
        else if (!row.get(last))
        {
            value2 = -value2;
        }
        
        //set the values found based on player color
        if (!color)
        {
            value = -value;
            value2 = -value2;
        }
        
        if (value == value2)
        {    
            return value;
        }
        else if (Math.abs(value) == Math.abs(value2))
        {
            return 0;
        }
        else if (Math.abs(value) > Math.abs(value2))
        {
            return value;
        }
        else
        {
            return value2;
        }
            
    }
}



class State
{
    Boolean[][] gameState = new Boolean[6][7];
    boolean color; //color of the player whose turn it is to move
    
    private ArrayList<State> subNodes = new ArrayList<State>();    
    
    static final int DEPTH=5;
    int move;
    int expectedMove;
    int value;
    
    public State(Boolean[][] gameState, boolean color)
    {
        for (int i = 0; i < gameState.length; i++) //necessary since cloning a multidimensional array will return a shallow clone
        {
            this.gameState[i] = gameState[i].clone();
        }
        this.color = color;  
        
        for (int i = 0; i < gameState[0].length; i++)
        {
            if ((gameState[0][i] == null))
            {
                subNodes.add(new State(this.gameState, !color, i, 0));
            }
        }            
    }
    
    public State(Boolean[][] gameState, boolean color, int nextMove, int currentDepth)
    {
        for (int i = 0; i < gameState.length; i++) //necessary since cloning a multidimensional array will return a shallow clone
        {
            this.gameState[i] = gameState[i].clone();
        }
        this.color = color;
        move = nextMove;
        
        insert(nextMove, !color);
        
        currentDepth++;        
        if (currentDepth < DEPTH)
        {            
            for (int i = 0; i < gameState[0].length; i++)
            {                
                if ((gameState[0][i] == null))
                {
                    subNodes.add(new State(this.gameState, !color, i, currentDepth));
                }
            }            
        }
    }
    
    public Boolean[][] getState()
    {
        return gameState;
    }
    
    public boolean getColor()
    {
        return color;
    }
    
    public void setValue(int value)
    {
        this.value = value;
    }
    
    public int getValue()
    {
        return value;
    }
    
    public int getMove()
    {
        return move;
    }
    
    public void setExpectedMove(int expected)
    {
        expectedMove = expected;
    }
    
    public int getExpectedMove()
    {
        return expectedMove;
    }
    
    public void insert(int column, boolean color)
    {
        for (int i = gameState.length-1; i >= 0; i--)
        {
            if (gameState[i][column] == null)
            {
                gameState[i][column] = new Boolean(color);
                i = 0;
            }
        }
    }
    
    public ArrayList<State> getSubNodes()
    {
        return subNodes;
    }
}
